<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use Carbon\Carbon;
use App\Models\ApiModels\RoomSeats;
use Exception;
use App\Models\ApiModels\StudentRegister;
use DB;
use App\Models\ApiModels\ExpenseDetails;
use App\Models\ApiModels\Expense;
use App\Models\ApiModels\Sales;
use App\Models\ApiModels\CustomeSales;
use App\Models\TanantComplaint;


class DashboardController extends Controller
{
    public function index()
    {
        try {
            $data = [];
            $totalRoom = RoomSeats::count();
            $totalSeats = RoomSeats::sum("numberseats");
            $engegedSeat = Studentregister::where("status", 1)->count();
            $totalEngagedRooms = RoomSeats::has('students', '>', 0)
                ->whereHas('students', function ($query) {
                    $query->where('status', 1);
                })
                ->select(DB::raw('COUNT(DISTINCT roomno) as total_engaged_rooms'))
                ->first();
            // Registrations
            $totalRegistration = StudentRegister::count();
            $currentMonthRegistration = StudentRegister::whereMonth('date', '=', now()->month)
                ->whereYear('date', '=', now()->year)
                ->count();
            // expenses records 
            $expensesByMonth = ExpenseDetails::select(DB::raw('SUM(amount) as total_amount'))
                ->whereRaw('MONTH(billdate) = ?', [now()->month])
                ->first();

            $totalExpense = Expense::join('expensedetails', 'expenses.id', '=', 'expensedetails.category_id')
                ->select(DB::raw('SUM(expensedetails.amount) as total'))
                ->value('total');
            $TotalExpenses = $totalExpense === "" ? 0 : $totalExpense;

            $ExpenseAmountMonthly = $expensesByMonth->total_amount ?? 0;

            $data[] = [
                'title' => 'Room Seats',
                "totalRoom" => $totalRoom,
                "totalSeats" => $totalSeats,
                "engagSeats" => $engegedSeat,
                'engagedRooms' => $totalEngagedRooms->total_engaged_rooms
            ];

            $data[] = [
                'title' => 'Registration',
                "TotalRegistration" => $totalRegistration,
                "currentMonthRegistration" => $currentMonthRegistration
            ];

            $data[] = [
                'title' => 'Expenses',
                'MonthlyExpenses' => $ExpenseAmountMonthly,
                'TotalExpenses' => $TotalExpenses
            ];

            return response()->json([
                'status' => true,
                "data" => $data
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                "status" => false,
                "error" => $e->getMessage(),
            ], 401);
        }
    }

    private function graphFilterValidation($request)
    {
        $validator = Validator::make($request->all(), [
            'filterType' => 'nullable|string',
            'startDate' => 'nullable|required_if:filterType,custom|date',
            'endDate' => 'nullable|required_if:filterType,custom|date'
        ]);
        return  $validator;
    }
    public function saleExpensesGraph(Request $request)
    {
        try {
            $validator =  $this->graphFilterValidation($request);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Validation errors',
                    'errors' => $validator->errors()
                ], 422);
            }

            $filterType = $request->filterType ?? "all";
            // Handle different filter types using separate methods
            switch ($filterType) {
                case 'year':
                    $salesData = Sales::select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('SUM(total) AS sale_total'))
                        ->groupBy(DB::raw('YEAR(invoicedate)'))
                        ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

                    $CustomSalesmonthlyExp = CustomeSales::select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('SUM(total) AS sale_total'))
                        ->groupBy(DB::raw('YEAR(invoicedate)'))
                        ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

                    $totalSalesData = $salesData->union($CustomSalesmonthlyExp)->get();

                    $expensesData = ExpenseDetails::select(DB::raw('YEAR(billdate) AS year_num'), DB::raw('SUM(amount) AS expenses_amount'))
                        ->groupBy(DB::raw('YEAR(billdate)'))
                        ->orderBy(DB::raw('YEAR(billdate)', 'asc'))
                        ->get();
                    $data = $this->processYearlyData($totalSalesData, $expensesData);
                    break;
                case 'quarter':
                    $currentYear = date('Y');
                    $currentMonth = date('m');
                    if ($currentMonth < 4) {
                        $startDate = ($currentYear - 1) . "-04-01";
                        $endDate = $currentYear . "-03-31";
                    } else {
                        $startDate = $currentYear . "-04-01";
                        $endDate = ($currentYear + 1) . "-03-31";
                    }            // Retrieve the quarterly sales data
                    $query = Sales::select(DB::raw('QUARTER(invoicedate) AS quarter_num'), DB::raw('SUM(total) AS total_amount'))
                        ->whereBetween('invoicedate', [$startDate, $endDate])
                        ->groupBy(DB::raw('QUARTER(invoicedate)'))
                        ->orderBy(DB::raw('QUARTER(invoicedate)'));

                    $CustomSalesmonthlyExp = CustomeSales::select(DB::raw('QUARTER(invoicedate) AS quarter_num'), DB::raw('SUM(total) AS total_amount'))
                        ->whereBetween('invoicedate', [$startDate, $endDate])
                        ->groupBy(DB::raw('QUARTER(invoicedate)'))
                        ->orderBy(DB::raw('QUARTER(invoicedate)'));

                    $saleAndCusomeSaleData = $query->union($CustomSalesmonthlyExp)->get();
                    // Retrieve the quarterly expenses data
                    $expensesData = ExpenseDetails::select(DB::raw('QUARTER(billdate) AS quarter_num'), DB::raw('SUM(amount) AS expenses_amount'))
                        ->whereBetween('billdate', [$startDate, $endDate])
                        ->groupBy(DB::raw('QUARTER(billdate)'))
                        ->orderBy(DB::raw('QUARTER(billdate)'))
                        ->get();
                    $data = $this->processQuarterlyData($saleAndCusomeSaleData, $expensesData);
                    break;
                case 'month':
                    $currentMonth = date('m');
                    $currentDay = date('d');
                    $currentYear = date('Y');
                    // Retrieve the daily sales data for the current month
                    $salesData = Sales::select(DB::raw('DATE_FORMAT(invoicedate, "%d") AS days'), 'total AS sales_amount')
                        ->whereMonth('invoicedate', '=', $currentMonth)
                        ->whereDay('invoicedate', '<=', $currentDay)
                        ->whereYear('invoicedate', '=', $currentYear);

                    $CustomSalesmonthlyExp = CustomeSales::select(DB::raw('DATE_FORMAT(invoicedate, "%d") AS days'), 'total AS sales_amount')
                        ->whereMonth('invoicedate', '=', $currentMonth)
                        ->whereDay('invoicedate', '<=', $currentDay);

                    $saleAndCusomeSaleData = $salesData->union($CustomSalesmonthlyExp)->get();
                    // Retrieve the daily expenses data for the current month
                    $expensesData = ExpenseDetails::select(DB::raw('DATE_FORMAT(billdate, "%d") AS days'), 'amount AS expenses_amount')
                        ->whereMonth('billdate', '=', $currentMonth)
                        ->whereDay('billdate', '<=', $currentDay)
                        ->get();
                    $data = $this->processMonthlyData($saleAndCusomeSaleData, $expensesData);
                    break;
                case 'Months':
                    $currentYear = date('Y');
                    $currentMonth = date('m');
                    if ($currentMonth < 4) {
                        $startDate = ($currentYear - 1) . "-04-01";
                        $endDate = $currentYear . "-03-31";
                    } else {
                        $startDate = $currentYear . "-04-01";
                        $endDate = ($currentYear + 1) . "-03-31";
                    }
                    $query = Sales::select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                        ->whereBetween('invoicedate', [$startDate, $endDate])
                        ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                        ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

                    $customSalesQuery = CustomeSales::select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                        ->whereBetween('invoicedate', [$startDate, $endDate])
                        ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                        ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

                    $saleAndCusomeSaleData = $query->unionAll($customSalesQuery)->get();

                    $ExpensesmonthlyExp = ExpenseDetails::select(DB::raw('YEAR(billdate) AS year_num'), DB::raw('MONTH(billdate) AS month_num'), DB::raw('SUM(amount) AS expenses_amount'))
                        ->whereBetween('billdate', [$startDate, $endDate])
                        ->groupBy(DB::raw('YEAR(billdate)'), DB::raw('MONTH(billdate)'))
                        ->orderBy(DB::raw('YEAR(billdate)'), 'ASC')
                        ->get();
                    $data = $this->processMonthsData($saleAndCusomeSaleData, $ExpensesmonthlyExp);
                    break;
                case 'custom':
                    $startDate = $request['startDate'];
                    $endDate = $request['endDate'];

                    $query = Sales::select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                        ->whereBetween('invoicedate', [$startDate, $endDate])
                        ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                        ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

                    $customSalesQuery = CustomeSales::select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                        ->whereBetween('invoicedate', [$startDate, $endDate])
                        ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                        ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

                    $saleAndCustomSaleData = $query->unionAll($customSalesQuery)->get();

                    $ExpensesmonthlyExp = ExpenseDetails::select(DB::raw('YEAR(billdate) AS year_num'), DB::raw('MONTH(billdate) AS month_num'), DB::raw('SUM(amount) AS expenses_amount'))
                        ->whereBetween('billdate', [$startDate, $endDate])
                        ->groupBy(DB::raw('YEAR(billdate)'), DB::raw('MONTH(billdate)'))
                        ->orderBy(DB::raw('YEAR(billdate)'), 'ASC')
                        ->get();
                    $data = $this->processCustomData($saleAndCustomSaleData, $ExpensesmonthlyExp);
                    break;
                default:
                    $SalesmonthlyExp = Sales::select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS sale_total'))
                        ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                        ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

                    $CustomSalesmonthlyExp =  CustomeSales::select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS sale_total'))
                        ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                        ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');
                    $saleAndCusomeSaleData = $SalesmonthlyExp->unionAll($CustomSalesmonthlyExp)->get();

                    $ExpensesmonthlyExp =  ExpenseDetails::select(DB::raw('DATE_FORMAT(billdate, "%m") AS month_num'), DB::raw('SUM(amount) AS expenses_amount'))
                        ->groupBy(DB::raw('DATE_FORMAT(billdate, "%m")'))
                        ->orderBy(DB::raw('DATE_FORMAT(billdate, "%m")'))
                        ->get();

                    // Initialize total amounts arrays
                    $salestotalAmount = array_fill(1, 12, 0);
                    $expensetotalAmount = array_fill(1, 12, 0);

                    // Populate total amounts arrays
                    foreach ($saleAndCusomeSaleData as $sales) {
                        $monthNum = intval($sales->month_num);
                        $salestotalAmount[$monthNum] += $sales->sale_total;
                    }

                    foreach ($ExpensesmonthlyExp as $expense) {
                        $monthNum = intval($expense->month_num);
                        $expensetotalAmount[$monthNum] = (int) $expense->expenses_amount;
                    }

                    // Generate the month labels and total amounts
                    for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                        $monthLabels[] = date("M", mktime(0, 0, 0, $monthNum, 1));
                    }

                    // Generate data array for chart
                    foreach ($salestotalAmount as $key => $value) {
                        $data[] = [
                            'value' => $value,
                            'frontColor' => '#ffc107',
                            'gradientColor' => '#ffc107',
                            'spacing' => 0,
                            'label' => $monthLabels[$key - 1],
                        ];

                        // Check if key exists in the expenses array before accessing it
                        if (array_key_exists($key, $expensetotalAmount)) {
                            $data[] = [
                                'value' => (int)$expensetotalAmount[$key],
                                'frontColor' => '#99ccff',
                                'gradientColor' => '#99ccff',
                            ];
                        }
                    }
            }
            return response()->json([
                'status' => true,
                'message' => "Data Loaded Successfully",
                'data' => $data
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    // Example method for processing yearly data
    private function processYearlyData($totalSalesData, $expensesData)
    {
        $labels = [];
        $salesTotal = [];
        $expensesTotal = [];
        // Process sales data
        foreach ($totalSalesData as $salesRow) {
            $yearNum = $salesRow->year_num;
            if (!in_array($yearNum, $labels)) {
                $labels[] = $yearNum;
            }
            if (!isset($salesTotal[$yearNum])) {
                $salesTotal[$yearNum] = round($salesRow->sale_total);
            } else {
                $salesTotal[$yearNum] += round($salesRow->sale_total);
            }
        }

        foreach ($expensesData as $expensesRow) {
            $yearNum = $expensesRow->year_num;
            if (!in_array($yearNum, $labels)) {
                $labels[] = $yearNum;
            }
            if (!isset($expensesTotal[$yearNum])) {
                $expensesTotal[$yearNum] = round($expensesRow->expenses_amount);
            } else {
                $expensesTotal[$yearNum] += round($expensesRow->expenses_amount);
            }
        }
        // Fill in missing years in expenses data with 0 total
        foreach ($salesTotal as $yearNum => $saleTotal) {
            if (!array_key_exists($yearNum, $expensesTotal)) {
                $expensesTotal[$yearNum] = 0;
            }
            if (!array_key_exists($yearNum, $salesTotal)) {
                $salesTotal[$yearNum] = 0;
            }
        }
        // Sort the data by year
        ksort($salesTotal);
        ksort($expensesTotal);

        foreach ($salesTotal as $key => $value) {
            $data[] = [
                'value' => $value,
                'frontColor' => '#ffc107',
                'gradientColor' => '#ffc107',
                'spacing' => 0,
                'label' => $key,
            ];

            // Check if key exists in the expenses array before accessing it
            if (array_key_exists($key, $expensesTotal)) {
                $data[] = [
                    'value' => (int)$expensesTotal[$key],
                    'frontColor' => '#99ccff',
                    'gradientColor' => '#99ccff',
                ];
            }
        }
        return $data;
    }

    // Example method for processing quarterly data
    private function processQuarterlyData($saleAndCusomeSaleData, $expensesData)
    {
        // Create an array to store the quarterly sales and expenses totals
        $salesTotal = array_fill(1, 4, 0); // Initialize all quarters with zero sales total
        $expensesTotal = array_fill(1, 4, 0); // Initialize all quarters with zero expenses total
        // Populate the arrays with the retrieved quarterly totals
        foreach ($saleAndCusomeSaleData as $row) {
            $quarterNum = $row->quarter_num;
            $salesTotal[$quarterNum] = round($row->total_amount);
        }
        foreach ($expensesData as $row) {
            $quarterNum = $row->quarter_num;
            $expensesTotal[$quarterNum] = round($row->expenses_amount);
        }
        // Generate the labels for the quarters
        $labels = ['Q1', 'Q2', 'Q3', 'Q4'];
        // dd($labels,$salesTotal,$expensesTotal);
        // Prepare the data for the response
        foreach ($salesTotal as $key => $value) {
            $data[] = [
                'value' => $value,
                'frontColor' => '#ffc107',
                'gradientColor' => '#ffc107',
                'spacing' => 0,
                'label' => $labels[$key - 1],
            ];
            // Check if key exists in the expenses array before accessing it
            if (array_key_exists($key, $expensesTotal)) {
                $data[] = [
                    'value' => (int)$expensesTotal[$key],
                    'frontColor' => '#99ccff',
                    'gradientColor' => '#99ccff',
                ];
            }
        }
        // Your logic here to fetch and process quarterly data
        return $data;
    }

    // Example method for processing monthly data
    private function processMonthlyData($saleAndCusomeSaleData, $expensesData)
    {
        $totalDays = Carbon::now()->daysInMonth;
        // Initialize the labels, sales amounts, and expenses amounts arrays
        $labels = [];
        $salesAmounts = array_fill(1, $totalDays, 0);
        $expensesAmounts = array_fill(1, $totalDays, 0);
        // Populate the labels array and calculate the sales and expenses amounts
        for ($day = 1; $day <= $totalDays; $day++) {
            $labels[] = str_pad($day, 2, '0', STR_PAD_LEFT);
        }
        foreach ($saleAndCusomeSaleData as $sale) {
            $dayNum = intval($sale->days);
            $salesAmounts[$dayNum] += round($sale->sales_amount);
        }
        foreach ($expensesData as $expense) {
            $dayNum = intval($expense->days);
            $expensesAmounts[$dayNum] += round($expense->expenses_amount);
        }
        // Populate total amounts arrays
        foreach ($salesAmounts as $key => $value) {
            $data[] = [
                'value' => $value,
                'frontColor' => '#ffc107',
                'gradientColor' => '#ffc107',
                'spacing' => 0,
                'label' => $labels[$key - 1],
            ];

            // Check if key exists in the expenses array before accessing it
            if (array_key_exists($key, $expensesAmounts)) {
                $data[] = [
                    'value' => (int)$expensesAmounts[$key],
                    'frontColor' => '#99ccff',
                    'gradientColor' => '#99ccff',
                ];
            }
        }
        return $data;
    }

    // Example method for processing Months data
    private function processMonthsData($saleAndCusomeSaleData, $ExpensesmonthlyExp)
    {
        $currentYear = date('Y');
        $currentMonth = date('m');
        if ($currentMonth < 4) {
            $startDate = ($currentYear - 1) . "-04-01";
            $endDate = $currentYear . "-03-31";
        } else {
            $startDate = $currentYear . "-04-01";
            $endDate = ($currentYear + 1) . "-03-31";
        }
        $query = Sales::select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
            ->whereBetween('invoicedate', [$startDate, $endDate])
            ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
            ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

        $customSalesQuery = CustomeSales::select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
            ->whereBetween('invoicedate', [$startDate, $endDate])
            ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
            ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

        $saleAndCusomeSaleData = $query->unionAll($customSalesQuery)->get();

        $ExpensesmonthlyExp = ExpenseDetails::select(DB::raw('YEAR(billdate) AS year_num'), DB::raw('MONTH(billdate) AS month_num'), DB::raw('SUM(amount) AS expenses_amount'))
            ->whereBetween('billdate', [$startDate, $endDate])
            ->groupBy(DB::raw('YEAR(billdate)'), DB::raw('MONTH(billdate)'))
            ->orderBy(DB::raw('YEAR(billdate)'), 'ASC')
            ->get();
        $salestotalAmount = [];
        $expensetotalAmount = [];
        // Initialize the total amount array for all months to 0
        $salestotalAmount = array_fill(1, 12, 0);
        $expensetotalAmount = array_fill(1, 12, 0);

        foreach ($saleAndCusomeSaleData as $expense) {
            $monthNum = intval($expense->month_num);
            $salestotalAmount[$monthNum] += round($expense->total_amount);
        }
        foreach ($ExpensesmonthlyExp as $expense) {
            $monthNum = intval($expense->month_num);
            $expensetotalAmount[$monthNum] = round($expense->expenses_amount);
        }
        // Generate the month labels and total amounts
        for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
            $monthLabels[] = date("M", mktime(0, 0, 0, $monthNum, 1));
        }

        foreach ($salestotalAmount as $key => $value) {
            $data[] = [
                'value' => $value,
                'frontColor' => '#ffc107',
                'gradientColor' => '#ffc107',
                'spacing' => 0,
                'label' => $monthLabels[$key - 1],
            ];

            // Check if key exists in the expenses array before accessing it
            if (array_key_exists($key, $expensetotalAmount)) {
                $data[] = [
                    'value' => (int)$expensetotalAmount[$key],
                    'frontColor' => '#99ccff',
                    'gradientColor' => '#99ccff',
                ];
            }
        }
        return $data;
    }

    // Example method for processing custom data
    private function processCustomData($result, $ExpensesmonthlyExp)
    {
        $monthLabels = [];
        $salestotalAmount = array_fill(1, 12, 0); // Initialize only if needed
        $expensetotalAmount = array_fill(1, 12, 0); // Initialize only if needed

        foreach ($result as $sale) {
            $monthNum = intval($sale->month_num);
            $salestotalAmount[$monthNum] += round($sale->total_amount);
        }

        foreach ($ExpensesmonthlyExp as $expense) {
            $monthNum = intval($expense->month_num);
            $expensetotalAmount[$monthNum] = round($expense->expenses_amount);
        }

        // Generate the month labels and total amounts
        for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
            $monthLabels[] = date("M", mktime(0, 0, 0, $monthNum, 1));
        }

        foreach ($salestotalAmount as $key => $value) {
            $data[] = [
                'value' => $value,
                'frontColor' => '#ffc107',
                'gradientColor' => '#ffc107',
                'spacing' => 0,
                'label' => $monthLabels[$key - 1],
            ];

            // Check if key exists in the expenses array before accessing it
            if (array_key_exists($key, $expensetotalAmount)) {
                $data[] = [
                    'value' => (int)$expensetotalAmount[$key],
                    'frontColor' => '#99ccff',
                    'gradientColor' => '#99ccff',
                ];
            }
        }
        return $data;
    }

    public function createSaleGraphData()
    {
        try {
            $labels = [];
            $totalAmount = [];
            $data = [];

            $query = Sales::select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $CustomSalesmonthlyExp = CustomeSales::select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $result = $query->unionAll($CustomSalesmonthlyExp)->get();

            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $labels[] = date("M", mktime(0, 0, 0, $monthNum, 1));
                $totalAmount[$monthNum] = 0;
            }
            // Sum up the total amounts for each month
            foreach ($result as $sales) {
                $monthNum = intval($sales->month_num);
                $totalAmount[$monthNum] += $sales->total_amount;
            }
            foreach ($totalAmount as $key => $value) {
                $data[] = [
                    'value' => $value,
                    'frontColor' => '#ffc107',
                    'gradientColor' => '#ffc107',
                    'spacing' => 6,
                    'label' => $labels[$key - 1],
                ];
            }

            return response()->json([
                'status' => true,
                'message' => "Data Loaded Successfully",
                'data' => $data
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function createExpenseGraphData()
    {
        try {
            $query = ExpenseDetails::select(DB::raw('YEAR(billdate) AS year_num'), DB::raw('MONTH(billdate) AS month_num'), DB::raw('SUM(amount) AS total_amount'))
                ->groupBy(DB::raw('YEAR(billdate)'), DB::raw('MONTH(billdate)'))
                ->orderBy(DB::raw('YEAR(billdate)'), 'asc');
            $result = $query->get();
            $data = [];
            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $labels[] = date("M", mktime(0, 0, 0, $monthNum, 1));
                $totalAmount[$monthNum] = 0;
            }
            // Sum up the total amounts for each month
            foreach ($result as $expense) {
                $monthNum = intval($expense->month_num);
                $totalAmount[$monthNum] = (float)($expense->total_amount);
            }

            foreach ($totalAmount as $key => $value) {
                $data[] = [
                    'value' => $value,
                    'frontColor' => '#99ccff',
                    'gradientColor' => '#99ccff',
                    'spacing' => 6,
                    'label' => $labels[$key - 1],
                ];
            }
            return response()->json([
                'status' => true,
                'message' => "Data Loaded Successfully",
                'data' => $data
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function expensesGraphWithCategoryName(Request $request)
    {
        try {
            $validator =  $this->graphFilterValidation($request);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Validation errors',
                    'errors' => $validator->errors()
                ], 422);
            }

            $filterType = $request->filterType ?? "all";

            $colors =  [
                '#99ccff',
                '#ffff4d',
                '#c2c2a3',
                '#ffc2ff',
                '#66ff66',
                '#85adad',
                '#6666ff',
                '#ffb84d',
                '#80002a',
                '#e67300',
                '#595959',
                '#004d4d',
                '#66004d',
                '#595959',
                '#4d004d',
                '#264d00',
                '#804000',
                '#3d3d5c',
                '#264d73',
                '#00664d',
            ];

            $data = [];
            $year = Carbon::now()->year;
            $currentMonth = Carbon::now()->month;
            $currentDay = Carbon::now()->day;

            switch ($filterType) {
                case 'year':
                    $expesnse = Expense::withCount(['details as amount' => function ($query) use ($year) {
                        $query->select(DB::raw('SUM(amount)'));
                        $query->whereYear('billdate', $year);
                    }])->get(['category_name as category']);
                    break;
                case 'quarter':
                    $expesnse = Expense::withCount(['details as amount' => function ($query) use ($year) {
                        $query->select(DB::raw('SUM(amount)'))
                            ->whereYear('billdate', $year)
                            ->orderBy(DB::raw('QUARTER(billdate)'));
                    }])->get(['category_name as category']);
                    break;
                case 'month':
                    $expesnse = Expense::withCount(['details as amount' => function ($query) use ($year, $currentMonth, $currentDay) {
                        $query->select(DB::raw('SUM(amount)'))
                            ->whereYear('billdate', $year)
                            ->whereMonth('billdate', $currentMonth)
                            ->whereDay('billdate', $currentDay)
                            ->orderBy(DB::raw('MONTH(billdate)'));
                    }])->get(['category_name as category']);
                    break;
                case 'Months':
                    if ($currentMonth < 4) {
                        $startDate = ($year - 1) . "-04-01";
                        $endDate = $year . "-03-31";
                    } else {
                        $startDate = $year . "-04-01";
                        $endDate = ($year + 1) . "-03-31";
                    }
                    $expesnse = Expense::withCount(['details as amount' => function ($query) use ($startDate, $endDate) {
                        $query->select(DB::raw('SUM(amount)'))
                            ->whereBetween('billdate', [$startDate, $endDate]);
                    }])->get(['category_name as category']);
                    break;
                case 'custom':
                    $startDate = $request->input('startDate');
                    $endDate = $request->input('endDate');
                    $expesnse = Expense::withCount(['details as amount' => function ($query) use ($startDate, $endDate) {
                        $query->select(DB::raw('SUM(amount)'))
                            ->whereBetween('billdate', [$startDate, $endDate]);
                    }])->get(['category_name as category']);
                    break;
                default:
                    $expesnse = Expense::withCount(['details as amount' => function ($query) {
                        $query->select(DB::raw('SUM(amount)'));
                    }])
                        ->get(['category_name as category']);
            }
            $totalAmount = $expesnse->sum('amount');
            foreach ($expesnse as $key => $row) {
                $percentage = $totalAmount != 0 ? ($row->amount / $totalAmount) * 100 : 0;
                if (isset($row->amount)) {
                    $data[] = [
                        'label' => $row->category_name,
                        'value' => isset($row->amount) ? (float)$row->amount : 0,
                        'color' => isset($colors[$key]) ? $colors[$key] : '#cccccc',
                        'text' => number_format($percentage, 2) . '%',
                    ];
                }
            }
            return response()->json([
                'status' => true,
                'message' => "Data Loaded Successfully",
                'data' => $data
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    private function prepareComplantGraphData($totalComplaints, $labels)
    {
        $data = [];
        foreach ($totalComplaints as $key => $value) {
            $data[] = [
                'value' => $value,
                'frontColor' => '#99ffff',
                'gradientColor' => '#99ffff',
                'spacing' => 6,
                'label' => $labels[$key],
            ];
        }
        return $data;
    }

    public function genrateComplainBoardGraph(Request $request)
    {
        try {
            $validator =  $this->graphFilterValidation($request);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Validation errors',
                    'errors' => $validator->errors()
                ], 422);
            }
            $filterType = $request->filterType ?? "all";
            $year = Carbon::now()->year;
            $currentMonth = Carbon::now()->month;
            $currentDay = Carbon::now()->day;

            switch ($filterType) {
                case 'year':
                    $query = TanantComplaint::select(DB::raw('YEAR(complaint_date) AS year_num'), DB::raw('COUNT(*) AS total_complaints'))
                        ->groupBy(DB::raw('YEAR(complaint_date)'))
                        ->orderBy(DB::raw('YEAR(complaint_date)'))
                        ->get();

                    $labels = [];
                    $totalComplaints = [];
                    $data = [];


                    foreach ($query as $row) {
                        $yearNum = $row->year_num;
                        if (!in_array($yearNum, $labels)) {
                            $labels[] = $yearNum;
                            $totalComplaints[$yearNum] = round($row->total_complaints);
                        } else {
                            $totalComplaints[$yearNum] += round($row->total_complaints);
                        }
                    }

                    // Sort the labels array
                    sort($labels);

                    // Fill in missing years with 0 total sales
                    foreach ($labels as $year) {
                        if (!isset($totalComplaints[$year])) {
                            $totalComplaints[$year] = 0;
                        }
                    }
                    // Sort the totalComplaints array by keys (years)
                    ksort($totalComplaints);
                    foreach ($totalComplaints as $key => $value) {
                        $data[] = [
                            'value' => $value,
                            'frontColor' => '#99ffff',
                            'gradientColor' => '#99ffff',
                            'spacing' => 6,
                            'label' => $key,
                        ];
                    }
                    break;
                case 'quarter':
                    if ($currentMonth < 4) {
                        $startDate = ($year - 1) . "-04-01";
                        $endDate = $year . "-03-31";
                    } else {
                        $startDate = $year . "-04-01";
                        $endDate = ($year + 1) . "-03-31";
                    }

                    $query = TanantComplaint::select(DB::raw('QUARTER(complaint_date) AS quarter_num'), DB::raw('COUNT(*) AS total_complaints'))
                        ->whereBetween('complaint_date', [$startDate, $endDate])
                        ->groupBy(DB::raw('QUARTER(complaint_date)'))
                        ->orderBy(DB::raw('QUARTER(complaint_date)'))
                        ->get();

                    $totalComplaints = array_fill(1, 4, 0); // Initialize all quarters with zero totals
                    // Populate the array with the retrieved quarterly totals
                    foreach ($query as $row) {
                        $quarterNum = $row->quarter_num;
                        $totalComplaints[$quarterNum] = round($row->total_complaints);
                    }
                    $labels = [
                        1 => 'Q1',
                        2 => 'Q2',
                        3 => 'Q3',
                        4 => 'Q4'
                    ];
                    $data = $this->prepareComplantGraphData($totalComplaints, $labels);
                    break;
                case 'month':
                    $query = TanantComplaint::select(DB::raw('DATE_FORMAT(complaint_date, "%d") AS days'), DB::raw('COUNT(*) AS total_complaints'))
                        ->whereMonth('complaint_date', '=', $currentMonth)
                        ->whereYear('complaint_date', $year)
                        ->get();

                    $totalDays = Carbon::now()->daysInMonth;

                    for ($day = 0; $day < $totalDays; $day++) {
                        $labels[$day] = $day + 1;
                        $totalComplaints[$day] = 0;
                    }

                    foreach ($query as $complaint) {
                        $day = intval($complaint->days);
                        $totalComplaints[$day] = $complaint->total_complaints;
                    }
                    $data = $this->prepareComplantGraphData($totalComplaints, $labels);

                    break;
                case 'Months':
                    $currentYear = date('Y');
                    $currentMonth = date('m');
                    if ($currentMonth < 4) {
                        $startDate = ($currentYear - 1) . "-04-01";
                        $endDate = $currentYear . "-03-31";
                    } else {
                        $startDate = $currentYear . "-04-01";
                        $endDate = ($currentYear + 1) . "-03-31";
                    }
                    $complaints = TanantComplaint::select(
                        DB::raw('YEAR(complaint_date) as year_num'),
                        DB::raw('MONTH(complaint_date) AS month_num'),
                        DB::raw('COUNT(*) AS total_complaints')
                    )
                        ->whereBetween('complaint_date', [$startDate, $endDate])
                        ->groupBy(DB::raw('YEAR(complaint_date)'), DB::raw('MONTH(complaint_date)'))
                        ->get();

                    $labels = [];
                    $totalComplaints = [];
                    $data = [];

                    // Initialize arrays for 12 months
                    for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                        $labels[$monthNum] = date("M", mktime(0, 0, 0, $monthNum, 1));
                        $totalComplaints[$monthNum] = 0;
                    }

                    // Populate total complaints array from the query result
                    foreach ($complaints as $complaint) {
                        $monthNum = intval($complaint->month_num) + 1;
                        $totalComplaints[$monthNum] = $complaint->total_complaints;
                    }
                    $data = $this->prepareComplantGraphData($totalComplaints, $labels);

                    break;
                case 'custom':
                    $startDate = $request['startDate'];
                    $endDate = $request['endDate'];

                    $complaints = TanantComplaint::select(
                        DB::raw('YEAR(complaint_date) as year_num'),
                        DB::raw('MONTH(complaint_date) AS month_num'),
                        DB::raw('COUNT(*) AS total_complaints')
                    )
                        ->whereBetween('complaint_date', [$startDate, $endDate])
                        ->groupBy(DB::raw('YEAR(complaint_date)'), DB::raw('MONTH(complaint_date)'))
                        ->get();

                    $labels = [];
                    $totalComplaints = [];
                    $data = [];

                    // Initialize arrays for 12 months
                    for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                        $labels[$monthNum] = date("M", mktime(0, 0, 0, $monthNum, 1));
                        $totalComplaints[$monthNum] = 0;
                    }

                    // Populate total complaints array from the query result
                    foreach ($complaints as $complaint) {
                        $monthNum = intval($complaint->month_num) + 1;
                        $totalComplaints[$monthNum] = $complaint->total_complaints;
                    }
                    $data = $this->prepareComplantGraphData($totalComplaints, $labels);
                    break;
                default:
                    $complaints = TanantComplaint::select(
                        DB::raw('YEAR(complaint_date) as year_num'),
                        DB::raw('MONTH(complaint_date) AS month_num'),
                        DB::raw('COUNT(*) AS total_complaints')
                    )
                        ->groupBy(DB::raw('YEAR(complaint_date)'), DB::raw('MONTH(complaint_date)'))
                        ->get();

                    $labels = [];
                    $totalComplaints = [];
                    $data = [];

                    // Initialize arrays for 12 months
                    for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                        $labels[] = date("M", mktime(0, 0, 0, $monthNum, 1));
                        $totalComplaints[$monthNum] = 0;
                    }

                    // Populate total complaints array from the query result
                    foreach ($complaints as $complaint) {
                        $monthNum = intval($complaint->month_num);
                        $totalComplaints[$monthNum] = $complaint->total_complaints;
                    }

                    foreach ($totalComplaints as $key => $value) {
                        $data[] = [
                            'value' => $value,
                            'frontColor' => '#99ffff',
                            'gradientColor' => '#99ffff',
                            'spacing' => 6,
                            'label' => $labels[$key - 1],
                        ];
                    }
            }



            return response()->json([
                'status' => true,
                'message' => "Data Loaded Successfully",
                'data' => $data
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function compaintGraphCategoryType(Request $request)
    {
        try {
            $validator =  $this->graphFilterValidation($request);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Validation errors',
                    'errors' => $validator->errors()
                ], 422);
            }
            $filterType = $request->filterType ?? "all";

            $colors =  [
                '#99ccff',
                '#ffff4d',
                '#c2c2a3',
                '#ffc2ff',
                '#66ff66',
                '#85adad',
                '#6666ff',
                '#ffb84d',
                '#80002a',
                '#e67300',
                '#595959',
                '#004d4d',
                '#66004d',
                '#595959',
                '#4d004d',
                '#264d00',
                '#804000',
                '#3d3d5c',
                '#264d73',
                '#00664d',
            ];

            switch ($filterType) {
                case 'year':
                    $complaintsByType = TanantComplaint::select('complaint_type', DB::raw('COUNT(*) AS count'))
                        ->groupBy('complaint_type')
                        ->get();
                    break;
                case 'quarter':
                    $currentYear = date('Y');
                    $currentMonth = date('m');
                    if ($currentMonth < 4) {
                        $startDate = ($currentYear - 1) . "-04-01";
                        $endDate = $currentYear . "-03-31";
                    } else {
                        $startDate = $currentYear . "-04-01";
                        $endDate = ($currentYear + 1) . "-03-31";
                    }

                    $complaintsByType = TanantComplaint::select('complaint_type', DB::raw('COUNT(*) AS count'))
                        ->whereBetween('complaint_date', [$startDate, $endDate])
                        ->groupBy('complaint_type')
                        ->get();
                    break;
                case 'month':
                    $currentMonth = date('m');
                    $currentYear = date('Y');
                    $complaintsByType = TanantComplaint::select(DB::raw('complaint_type'), DB::raw('COUNT(*) AS count'))
                        ->whereMonth('complaint_date', '=', $currentMonth)
                        ->whereYear('complaint_date', '=', $currentYear)
                        ->groupBy('complaint_type')
                        ->get();
                    break;
                case 'Months':
                    $currentYear = date('Y');
                    $currentMonth = date('m');
                    if ($currentMonth < 4) {
                        $startDate = ($currentYear - 1) . "-04-01";
                        $endDate = $currentYear . "-03-31";
                    } else {
                        $startDate = $currentYear . "-04-01";
                        $endDate = ($currentYear + 1) . "-03-31";
                    }
                    $complaintsByType = TanantComplaint::select(
                        'complaint_type',
                        DB::raw('COUNT(*) AS count')
                    )
                        ->whereBetween('complaint_date', [$startDate, $endDate])
                        ->groupBy('complaint_type')
                        ->get();
                    break;
                case 'custom':
                    $startDate = $request['startDate'];
                    $endDate = $request['endDate'];
                    $complaintsByType = TanantComplaint::select(
                        'complaint_type',
                        DB::raw('COUNT(*) AS count')
                    )
                        ->whereBetween('complaint_date', [$startDate, $endDate])
                        ->groupBy('complaint_type')
                        ->get();
                        break;
                default:
                    $complaintsByType = TanantComplaint::select('complaint_type', DB::raw('COUNT(*) as count'))
                        ->groupBy('complaint_type')
                        ->get();
            }
            $data = [];
            $totalCount = $complaintsByType->sum('count');

            foreach ($complaintsByType as $key => $complaint) {
                $percentage = $totalCount != 0 ? ($complaint->count / $totalCount) * 100 : 0;

                $data[] = [
                    'label' => $complaint->complaint_type,
                    'value' => $complaint->count,
                    'color' => isset($colors[$key]) ? $colors[$key] : '#cccccc',
                    'text' => number_format($percentage, 2) . '%',
                ];
            }

            return response()->json([
                'status' => true,
                'message' => "Data Loaded Successfully",
                'data' => $data
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }
    public function calucalteProfitAndLoss()
    {
        try {
            $Sale = Sales::sum('total');
            $CustomeSale  = CustomeSales::sum('total');
            $totalExpense = ExpenseDetails::sum('amount');
            $totalSale = $Sale + $CustomeSale;
            $data = [
                [
                    'title' => 'Sale',
                    'value' => $totalSale,
                    'icon' => 'receipt'
                ],
                [
                    'title' => 'Expense',
                    'value' => $totalExpense,
                    'icon' => 'hand-holding-dollar'
                ]
            ];

            $totalcheck = $totalSale - $totalExpense;
            if ($totalcheck < 0) {
                $data[] = [
                    'title' => 'arrow-up-long',
                    'value' => abs($totalcheck),
                    'icon' => 'arrow-up-long',
                    'color' => '#ffc107',
                ];
                $data[] = [
                    'title' => 'arrow-up-long',
                    'value' => abs($totalcheck),
                    'icon' => 'hand-holding-dollar',
                    'color' => '#ffc107',
                ];
            } else {
                $data[] = [
                    'title' => 'Gross Profit',
                    'value' => $totalcheck,
                    'icon' => 'arrow-up-long',
                    'color' => '#198754',
                ];
                $data[] = [
                    'title' => 'Net Profit',
                    'value' => $totalcheck,
                    'icon' => 'hand-holding-dollar',
                    'color' => '#198754',
                ];
            }
            return response()->json([
                'status' => true,
                'message' => "Data Available",
                "data" => $data,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }
}
