<?php

namespace App\Http\Controllers\Api;

use Illuminate\Support\Facades\Validator;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Carbon\Carbon;
use Illuminate\Validation\Rule;
use Exception;
use App\Models\EmployeRegister;
use Illuminate\Support\Facades\DB;
use App\Models\EmpAttendance;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Validation\ValidationException;
use Symfony\Component\HttpKernel\Exception\HttpException;
use App\Models\Businessregister;

class EmpAttendanceController extends Controller
{
    private function getBusinessDetails()
    {
        $business = Businessregister::select('license_no', 'email', 'mobile')->firstOrFail();
        return [
            'license_no' => $business->license_no,
            'email' => $business->email,
            'mobile' => $business->mobile,
        ];
    }
    public function getEmpAttendace($empId)
    {
        try {

            $attendanceRecords = EmpAttendance::where('employee_id', $empId)->get();
            return response()->json([
                'status' => true,
                'message' => "Employee Attendance Records Fetched Successfully",
                'data'   => $attendanceRecords
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'An error occurred! Please try again later.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function calculateTotalAttendance($empId)
    {
        try {
            $attendanceData = EmpAttendance::where('employee_id', $empId)
                ->selectRaw('COUNT(CASE WHEN status = "P" THEN 1 END) as totalPresent')
                ->selectRaw('COUNT(CASE WHEN status = "A" THEN 1 END) as totalAbsent')
                ->selectRaw('COUNT(CASE WHEN status = "L" THEN 1 END) as totalLeave')
                ->selectRaw('COUNT(CASE WHEN status = "H" THEN 1 END) as totalHalf')
                ->first();

            if (!$attendanceData) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 404,
                    'message' => 'Attendance data not found for the employee',
                ], 404);
            }

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Attendance data calculated successfully',
                'data' => [
                    'totalPresent' => $attendanceData->totalPresent,
                    'totalAbsent' => $attendanceData->totalAbsent,
                    'totalLeave' => $attendanceData->totalLeave,
                    'totalHalf' => $attendanceData->totalHalf,
                ]
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'An error occurred! Please try again later.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function employeeAttendances(Request $request)
    {
        try {
            $businessData = $this->getBusinessDetails();
            $liceneNo = $businessData['license_no'];
            $selectedDate = $request->month ? Carbon::parse($request->month) : Carbon::now();
            $selectedMonth = $selectedDate->month;
            $selectedYear = $selectedDate->year;

            $data = EmployeRegister::with(['employeAttendance' => function ($query) use ($selectedMonth, $selectedYear) {
                $query->whereMonth('date', $selectedMonth)
                    ->whereYear('date', $selectedYear);
            }])->where('license_no', $liceneNo)->get();
            // $data->makeHidden(['companyDetails']);
            $data->each(function ($employee) use ($selectedMonth, $selectedYear, $selectedDate) {
                $data = $employee->everyMonthSalaryCalulation($selectedDate);
                $employee->attendanceForCurrentMonth = $data;
                $attendanceMetrics = $employee->calculateAttendance($selectedMonth, $selectedYear);
                $employee->totalPresent = $attendanceMetrics['totalPresent'];
                $employee->totalAbsent = $attendanceMetrics['totalAbsent'];
                $employee->totalLeave = $attendanceMetrics['totalLeave'];
                $employee->totalHalfDay = $attendanceMetrics['totalHalfDay'];
            });

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Employee attendances fetched successfully',
                'data' => $data,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'An error occurred! Please try again later.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function storeAttendance(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'attendance_date' => 'required|date',
                'data' => 'required|array|min:1',
                'data.*.status' => 'required|in:P,A,H,L',
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            // Begin a transaction to ensure atomicity
            \DB::beginTransaction();

            // Check if attendance already registered for the attendance_Date
            $checkAttendance = EmpAttendance::where('date', $request->attendance_date)->count();
            if ($checkAttendance > 0) {
                \DB::rollBack(); // Rollback the transaction
                return response()->json([
                    'status' => false,
                    'statusCode' => 409,
                    'message' => "Attendance already registered for $request->attendance_date"
                ], 409);
            }

            // Store attendance data for each employee
            foreach ($request->data as $attendance) {
                EmpAttendance::create([
                    'date' => $request->attendance_date,
                    'employee_id' => $attendance['id'],
                    'status' => $attendance['status'],
                ]);
            }
            \DB::commit();
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Attendance data stored successfully',
            ], 200);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 422,
                'message' => 'Validation error',
                'error' => $e->validator->errors(),
            ], 422);
        } catch (Exception $e) {
            \DB::rollBack();
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'An error occurred! Please try again later.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function updateAttendance(Request $request, $attendanceId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'status' => 'required|string|in:P,A,H,L'
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            // Check if attendance record exists
            $attendanceRecord = EmpAttendance::findOrFail($attendanceId);

            // Update attendance status
            $data = $request->data;
            $attendanceRecord->update([
                'status' => $request->status
            ]);

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Attendance data updated successfully',
            ], 200);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 422,
                'message' => 'Validation error',
                'error' => $e->validator->errors(),
            ], 422);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => 'Record not found',
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'An error occurred! Please try again later.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function deleteAttendance($attendanceId)
    {
        try {
            $attendanceRecord = EmpAttendance::findOrFail($attendanceId);
            $attendanceRecord->delete();

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Attendance data deleted successfully',
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => 'Record not found',
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'An error occurred! Please try again later.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
    public function addAttendance(Request $request, $empId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'date' => 'required|date',
                'status' => 'required|in:P,A,H,L',
            ]);
            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            $user = EmployeRegister::findOrFail($empId);

            $existingAttendance = EmpAttendance::where('date', $request->date)
                ->where('employee_id', $empId)
                ->first();

            if ($existingAttendance) {
                throw new Exception('Attendance already added for this date and employee.');
            }

            EmpAttendance::create([
                'date' => $request->date,
                'employee_id' => $empId,
                'status' => $request->status,
            ]);

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Attendance add  successfully',
            ], 200);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 422,
                'message' => 'Validation error',
                'error' => $e->validator->errors(),
            ], 422);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => 'Record not found',
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'An error occurred! Please try again later.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
}
