<?php

namespace App\Http\Controllers\Api;

use Illuminate\Support\Facades\Validator;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Carbon\Carbon;
use App\Models\empPaymentSetup;
use Exception;
use App\Models\EmpAttendance;
use App\Models\EmpSalary;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Validation\ValidationException;
use Symfony\Component\HttpKernel\Exception\HttpException;

class EmpSalarayManagementController extends Controller
{

    public function checkMonthlySalary(Request $request, $empId)
    {
        try {
            // Validate the request
            $validator =  Validator::make($request->all(), [
                'month_year' => ['required', 'date'],
            ]);
            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            // Extract input data
            $selectedDate = $request->input('month_year');
            $firstDayOfMonth = Carbon::parse($selectedDate)->startOfMonth();
            $lastDayOfMonth = Carbon::parse($selectedDate)->endOfMonth();

            // Fetch salary setup data for the employee
            $salarySetupData = EmpPaymentSetup::where('emp_id', $empId)->orderByDesc('id')->first();

            if (!$salarySetupData) {
                return response()->json([
                    'status' => false,
                    'message' => 'Payment setup not found.',
                ], 404);
            }

            // Extract necessary details from salary setup
            $leaveAllowed = $salarySetupData->leave_allowed;
            $selectedMonthsSalary = $salarySetupData->monthly_payment;

            // Fetch attendance data for the current month
            $attendanceData = EmpAttendance::where('employee_id', $empId)
                ->whereBetween('date', [$firstDayOfMonth, $lastDayOfMonth])
                ->get();

            // Initialize counters
            $totalPresent = 0;
            $totalAbsent = 0;
            $totalHalfDay = 0;
            $totalLeave = 0;

            // Process attendance records
            foreach ($attendanceData as $attendance) {
                switch ($attendance->status) {
                    case 'P':
                        $totalPresent++;
                        break;
                    case 'A':
                        $totalAbsent++;
                        break;
                    case 'H':
                        $totalHalfDay++;
                        break;
                    case 'L':
                        $totalLeave++;
                        break;
                    default:
                        // Handle unrecognized status if needed
                        break;
                }
            }
            // Adjust totalPresent based on half-days, absent days, and leave days
            $totalPresent += $totalHalfDay / 2;

            // Calculate the actual leave days based on the allowed limit
            if ($totalLeave > $leaveAllowed) {
                $actualLeave = $totalLeave - $leaveAllowed;
            } else {
                $actualLeave = 0;
            }

            // Deduct excess leave days beyond the allowed limit from totalPresent
            $totalPresent -= $totalAbsent + $actualLeave;

            // Ensure totalPresent is not negative
            $totalPresent = max($totalPresent, 0);

            // Calculate total salary based on present days
            $totalDaysInMonth = $lastDayOfMonth->diffInDays($firstDayOfMonth) + 1;
            $totalSalary = ($selectedMonthsSalary / $totalDaysInMonth) * $totalPresent;

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'salary details',
                'totalPresent' => round(max($totalPresent, 0), 2),
                'totalSalary' =>  round(max($totalSalary, 0), 2),
                'paymentDetails' => $this->getPaymentDetails($empId, $selectedDate),
            ], 200);
        } catch (ValidationException $e) {
            // Handle validation errors
            return response()->json([
                'status' => false,
                'statusCode' => 422,
                'message' => 'Validation error',
                'error' => $e->validator->errors(),
            ], 422);
        } catch (Exception $e) {
            // Handle other exceptions
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'An error occurred! Please try again later.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    private function  getPaymentDetails($empId, $selectedDate)
    {
        try {
            $slectedMonth = Carbon::parse($selectedDate)->month;
            $year = Carbon::parse($selectedDate)->year;
            $checkData = EmpSalary::where('emp_id', $empId)->whereMonth('salary_month', $slectedMonth)
                ->whereYear('salary_month', $year)
                ->first();
            return  [
                'data' => $checkData,
                'paymentStatus' => empty($checkData) ? 'Unpaid' : 'Paid'
            ];
        } catch (Exception $e) {
            dd($e->getMessage());
        }
    }

    public function deleteSalaryRecords($salaryId)
    {
        try {
            $empSalaryRecords =  EmpSalary::findOrFail($salaryId);
            $empSalaryRecords->delete();
            return response()->json([
                "status" => true,
                "statusCode" => 200,
                "message" => "Record deleted successfully",
            ]);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => 'Records not Found',
                'error' => $e->getMessage()
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'An error occurred! Please try again later.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function createSalaryRecords(Request $request, $empId)
    {
        try {

            $validator = Validator::make($request->all(), [
                'salary_month' => 'required|date',
                'pay_date' => 'required|date',
                'total_present' => 'required|numeric',
                'amount' => 'required|numeric',
                'bonus' => 'nullable|numeric',
                'penalty' => 'nullable|numeric',
                'total_amount' => 'required|numeric',
                'payment_type' => 'required|string',
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }
            // check if the salary all ready  paid for this month or not
            $salaryMonth = Carbon::parse($request->salary_month)->format('Y-m-d');
            $isPaid = EmpSalary::where('salary_month', $salaryMonth)->where('emp_id', $empId)->first();
            if (!empty($isPaid)) {
                throw new HttpException(409, 'Salary already created for this month');
            }
            $data = $validator->validated();
            $data['emp_id'] = $empId;
            $data['salary_month'] = $salaryMonth;
            EmpSalary::create($data);

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Payment Record created'
            ], 200);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 422,
                'message' => 'Validation error',
                'error' => $e->validator->errors(),
            ], 422);
        } catch (HttpException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 409,
                'message' => 'Salary already created for this month',
                'error' => $e->getMessage(),
            ], 409);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'An error occurred! Please try again later.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function getSalaryRecordsById($salaryId)
    {
        try {
            $salaryRecord = EmpSalary::findOrFail($salaryId);
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Salary records fetched successfully',
                "data" => $salaryRecord
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => 'Records not Found',
                'error' => $e->getMessage()
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'error' => $e->getMessage(),
                'message' => 'Somthing Went Wronge'
            ], 500);
        }
    }

    public function updateSalaryRecords(Request $request, $salaryId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'salary_month' => 'required|date',
                'pay_date' => 'required|date',
                'total_present' => 'required|numeric',
                'amount' => 'required|numeric',
                'bonus' => 'nullable|numeric',
                'penalty' => 'nullable|numeric',
                'total_amount' => 'required|numeric',
                'payment_type' => 'required|string',
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }
            $salaryRecord = EmpSalary::findOrFail($salaryId);
            $validatedData = $validator->validated();
            $salaryRecord->update($validatedData);

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'salary records updated successfully'
            ], 200);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 422,
                'message' => 'Validation error',
                'error' => $e->validator->errors(),
            ], 422);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => 'Records not Found',
                'error' => $e->getMessage()
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'An error occurred! Please try again later.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function getPaymentSetupData($empId)
    {
        try {
            $setupData = empPaymentSetup::where('emp_id', $empId)->get();
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Salary setup fetched successfully',
                "data" => $setupData
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'error' => $e->getMessage(),
                'message' => 'Somthing Went Wronge'
            ], 500);
        }
    }

    public function createPaymentSetup(Request $request, $empId)
    {
        try {
            // Validation
            $validator = Validator::make($request->all(), [
                'date' => 'required|date',
                'monthly_payment' => 'required|numeric',
                'leave_allowed' => 'required|numeric'
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            // Extract validated data and add emp_id
            $data = $validator->validated();
            $data['emp_id'] = $empId;

            // Create the salary setup record
            $salarySetup = EmpPaymentSetup::create($data);

            return response()->json([
                'status' => true,
                'statusCode' => 201,
                'message' => 'Employee Salary Setup Created Successfully!',
            ], 201);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 422,
                'message' => 'Validation error',
                'error' => $e->validator->errors(),
            ], 422);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'An error occurred! Please try again later.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
}
