<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use App\Models\ApiModels\StudentRegister;
use Exception;
use Illuminate\Validation\ValidationException;
use App\Models\Businessregister;
use App\Models\ApiModels\RoomSeats;
use App\Models\Sale;
use App\Models\ApiModels\StudentBooking;
use DB;
use Illuminate\Support\Facades\Log;
use App\Models\ApiModels\SelfStudentRegister;
use FFI\Exception as FFIException;
use App\Models\ApiModels\OldStudentsRegister;


class RoomController extends controller
{
    public function getRooms()
    {
        try {
            $roomData = [];
            $rooms = RoomSeats::all();

            if ($rooms->isEmpty()) {
                throw new Exception("No room data found.");
            }

            foreach ($rooms as $room) {
                $students = [];
                for ($i = 1; $i <= $room->numberseats; $i++) {
                    $studentData = $this->getStudentData($room->roomno, $i);
                    $students[] = !empty($studentData) ? $this->formatStudentData($studentData) : [];
                }
                $roomData[$room->roomno] = [
                    'id' => $room->id,
                    'roomtype' => $room->roomtype,
                    'seats' => $room->numberseats,
                    'nocandidate' => $room->nocandidate,
                    'rent' => $room->rent,
                    'totalStudents' => $students
                ];
            }

            return response()->json([
                'status' => true,
                'data' => $roomData,
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ]);
        }
    }
    public function deleteRoom($roomno)
    {
        try {
            $deleteRoom = RoomSeats::where("roomno", $roomno)->first();
            $studentsInRoom = Studentregister::where('roomno', $deleteRoom->roomno)->count();

            if ($studentsInRoom > 0) {
                return response()->json([
                    "status" => false,
                    "message" => "Cannot delete room {$roomno} seat because there are students assigned to it ",
                ], 422);
            }

            if ($deleteRoom) {
                $deleteRoom->delete();
                return response()->json([
                    "status" => true,
                    "message" => "Successfully deleted the room: " . $roomno,
                ]);
            } else {
                throw new Exception("Invalid Room number");
            }
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                "error" => $e->getMessage(),
            ]);
        }
    }
    public function createRoom(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'roomNo' => 'required',
                'roomType' => 'required',
                'seatNo' => 'required',
                'noOfCandidate' => 'required',
                'rent' => 'required',
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            // Check for duplicate room number
            if (RoomSeats::where('roomno', $request['roomNo'])->exists()) {
                throw new Exception('The room already exists: ' . $request->roomNo);
            }

            // Create a new room
            $newRoom = new RoomSeats();
            $newRoom->roomno = $request->roomNo;
            $newRoom->roomtype = $request->roomType;
            $newRoom->numberseats = $request->seatNo;
            $newRoom->nocandidate = $request->noOfCandidate;
            $newRoom->rent = $request->rent;

            if ($newRoom->save()) {
                return response()->json([
                    'status' => true,
                    'message' => 'Successfully created a new room with room number: ' . $request->roomNo,
                ], 201);
            } else {
                throw new Exception('Something went wrong while saving the room.');
            }
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'errors' => $e->errors(),
            ], 422);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ]);
        }
    }

    public function updateRoom(Request $request, $roomno)
    {
        try {
            $validator = Validator::make($request->all(), [
                'roomNo' => 'required',
                'roomType' => 'required',
                'seatNo' => 'required',
                'noOfCandidate' => 'required',
                'rent' => 'required',
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            // Find the existing room
            $existingRoom = RoomSeats::where("roomno", $roomno)->first();

            if (!$existingRoom) {
                throw new Exception('Room not found with room number: ' . $roomno);
            }

            // Update the room details
            $existingRoom->roomtype = $request->roomType;
            $existingRoom->numberseats = $request->seatNo;
            $existingRoom->nocandidate = $request->noOfCandidate;
            $existingRoom->rent = $request->rent;

            // Save the changes
            if ($existingRoom->update()) {
                return response()->json([
                    'status' => true,
                    'message' => 'Successfully updated room: ' . $roomno,
                ], 201);
            } else {
                throw new Exception('Something went wrong while saving the room.');
            }
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'errors' => $e->errors(),
            ], 422);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ]);
        }
    }

    public function getRoomsDetails()
    {
        try {
            $totalRoom = RoomSeats::count();
            $totalSeats = RoomSeats::sum("numberseats");
            $engegedSeat = Studentregister::where("status", 1)->count();

            return response()->json([
                'status' => true,
                "data" => [
                    [
                        "title" => "Total Room",
                        'value' => $totalRoom,
                    ],
                    [
                        "title" => "Total Seat",
                        'value' => $totalSeats,
                    ],
                    [
                        "title" => "Engeged Seat",
                        'value' => $engegedSeat,
                    ],
                    [
                        "title" => "Vacant Seats",
                        'value' => ($totalSeats - $engegedSeat)
                    ]
                ]
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ]);
        }
    }

    public function getStudentsData()
    {
        try {
            $registrationData = [];
            $rooms = RoomSeats::orderBy('roomno', 'asc')->get();

            if ($rooms->isEmpty()) {
                return response()->json([
                    'status' => false,
                    'error' => 'Rooms are not created.'
                ]);
            }

            foreach ($rooms as $room) {
                $totalSeats = $room->numberseats;
                $students = [];
                // $bookings = [];

                for ($i = 1; $i <= $totalSeats; $i++) {
                    $studentData = $this->getStudentData($room->roomno, $i);
                    $bookingData = $this->getBookingData($room->roomno, $i);
                    $data = [];
                    $BothData = [];
                    $data['reg'] = !empty($studentData) ? $this->formatStudentData($studentData) : [];
                    $data['book'] = !empty($bookingData) ? $this->formatBookingData($bookingData) : [];
                    $students[] = array_merge($data, $BothData);
                }
                $registrationData[] = [
                    'roomNo' => $room->roomno,
                    'roomType' => $room->roomtype,
                    'totalSeats' => $totalSeats,
                    'rent' => $room->rent,
                    'studentData' => $students,
                ];
            }

            return response()->json([
                'status' => true,
                'data' => $registrationData
            ]);
        } catch (Exception $e) {
            Log::error('Error in getStudentsData: ' . $e->getMessage());
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    private function getStudentData($roomNo, $seatNo)
    {
        return StudentRegister::where('roomno', $roomNo)
            ->where('seetno', $seatNo)
            ->where('status', 1)
            ->first();
    }

    private function getBookingData($roomNo, $seatNo)
    {
        return StudentBooking::where('roomno', $roomNo)
            ->where('seatno', $seatNo)
            ->where('status', 1)
            ->first();
    }

    private function formatStudentData($studentData)
    {
        // Format your student data here 
        return [
            'id' => $studentData->id,
            'registrationNumber' => $studentData->formno,
            'name' => $studentData->candidatename,
            'mobileNumber' => $studentData->phone,
            'seatNumber' => $studentData->seetno,
            'registrationDate' => $studentData->date,
            'bithDate' => $studentData->dob,
            'aadharNumber' => $studentData->valid,
            'email' => $studentData->email,
            'bloodGroup' => $studentData->blood,
            'helthIssue' => $studentData->healthissue,
            'healthDescription' => $studentData->health_description,
            'stayDuration' => $studentData->stydrt,
            'vehicalNumber' => $studentData->vehicalno,
            'vehicleDescription' => $studentData->vehicledescription,
            'courseName' => $studentData->coursename,
            'courseDescription' => $studentData->coursedetails,
            'jobdDescription' => $studentData->jobdetails,
            'instituteCompany' => $studentData->institute,
            'companyName' => $studentData->companyname,
            'instituteName' => $studentData->institutename,
            'fatherName' => $studentData->fathername,
            'fatherOccupation' => $studentData->fatheroocc,
            'motherName' => $studentData->mothername,
            'motherOccupation' => $studentData->motheroocc,
            'parentMobile1' => $studentData->tel,
            'parentMobile2' => $studentData->mobilenumber,
            'parentEmail' => $studentData->pemail,
            'parentAddress' => $studentData->fullAddress,
            'state' => $studentData->state,
            'pincode' => $studentData->pinno,
            'guardianName' => $studentData->guardianname,
            'guardianMobileNumber' => $studentData->guardiannumber,
            'guardianAddress' => $studentData->guardianaddress,
            'CandidateImage' => $studentData->uploadimage ? asset('assets/uploads/' . $studentData->uploadimage) : null,
            'aadharFrount' =>   $studentData->uploadaadharfront ? asset('assets/uploads/' . $studentData->uploadaadharfront) : null,
            'AadharBack' => $studentData->uploadaadharback ? asset('assets/uploads/' . $studentData->uploadaadharback) : null,
            'candidateSignature' => $studentData->candidatesign ? asset('assets/uploads/' . $studentData->candidatesign) : null,
            'ParentSignature' => $studentData->Parentsign ? asset('assets/uploads/' . $studentData->Parentsign) : null,
            'remainingAmount' => $this->calculateRemainingAmount($studentData->id),
            'registrationsDate' => $studentData->student_reg_date
        ];
    }

    private function formatBookingData($bookingData)
    {
        // Format your booking data here 
        return [
            'id' => $bookingData->id,
            'name' => $bookingData->candi_name,
            'seatNumber' => $bookingData->seatno,
            'mobileNumber' => $bookingData->candidate_mobile,
            'bookingDate' => $bookingData->booking_date,
            'registrationDate' => $bookingData->regdate,
            'BookingAmount' => $bookingData->Booking_amount,
        ];
    }

    private function calculateRemainingAmount($studentId)
    {

        $totalAmount = Sale::where('studentid', $studentId)
            ->select(DB::raw('SUM(remaining_amount) as amount'))
            ->first();

        return !empty($totalAmount) ? number_format($totalAmount->amount, 2) : 0;
    }

    public function getSelfStudentRegisters()
    {
        try {
            $studentData = [];
            $selfStudents = SelfStudentRegister::all();
            if (empty($selfStudents)) {
                throw new Exception("Records is not found");
            }
            $selfStudentData = [];
            foreach ($selfStudents as $studentData) {
                $selfStudentData[] = [
                    'id' => $studentData->id,
                    'registrationDate' => $studentData->regdate,
                    'name' => $studentData->candidatename,
                    'mobileNumber' => $studentData->phone,
                    'bithDate' => $studentData->dob,
                    'aadharNumber' => $studentData->aadhar,
                    'email' => $studentData->email,
                    'bloodGroup' => $studentData->blood,
                    'helthIssue' => $studentData->healthissue,
                    'healthDescription' => $studentData->health_description,
                    'stayDuration' => $studentData->stydrt,
                    'vehicalNumber' => $studentData->vehicalno,
                    'vehicleDescription' => $studentData->vehicledescription,
                    'courseName' => $studentData->course_job,
                    'courseDescription' => $studentData->coursedetails,
                    'jobdDescription' => $studentData->jobdetails,
                    'instituteCompany' => $studentData->institute_company,
                    'companyName' => $studentData->companyname,
                    'instituteName' => $studentData->institutename,
                    'fatherName' => $studentData->fathername,
                    'fatherOccupation' => $studentData->fatheroocc,
                    'motherName' => $studentData->mothername,
                    'motherOccupation' => $studentData->motheroocc,
                    'parentMobile1' => $studentData->parentmobile1,
                    'parentMobile2' => $studentData->parentmobile2,
                    'parentEmail' => $studentData->pemail,
                    'parentAddress' => $studentData->fullAddress,
                    'state' => $studentData->state,
                    'pincode' => $studentData->pinno,
                    'guardianName' => $studentData->guardianname,
                    'guardianMobileNumber' => $studentData->guardiannumber,
                    'guardianAddress' => $studentData->guardianaddress,
                    'CandidateImage' => $studentData->studentImg ? asset('assets/uploads/' . $studentData->studentImg) : null,
                    'aadharFrount' =>   $studentData->uploadaadharfront ? asset('assets/uploads/' . $studentData->uploadaadharfront) : null,
                    'AadharBack' => $studentData->uploadaadharback ? asset('assets/uploads/' . $studentData->uploadaadharback) : null,
                    'candidateSignature' => $studentData->candidatesign ? asset('assets/uploads/' . $studentData->candidatesign) : null,
                    'ParentSignature' => $studentData->Parentsign ? asset('assets/uploads/' . $studentData->Parentsign) : null,
                    'termsAndCondition' => $studentData->termsAndCondition,
                ];
            }
            return response()->json([
                "status" => true,
                'data' => $selfStudentData,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                "error" => $e->getMessage(),
            ], 500);
        }
    }

    private function getRoomRent($roomno)
    {
        try {
            $roomRent = RoomSeats::select("rent")->where("roomno", $roomno)->first();
            if (!empty($roomRent)) {
                return $roomRent->rent;
            }
        } catch (Exception $e) {
            return null;
        }
    }

    private function getAmount($formno)
    {
        try {
            $studentId = StudentRegister::select("id")->where('formno', $formno)->first();
            if (!empty($studentId)) {
                $amount = $this->calculateRemainingAmount($studentId->id);
                return $amount;
            }
        } catch (Exception $e) {
            return null;
        }
    }

    private function getActualStudentId($studentFormNo)
    {
        $studentId = StudentRegister::where('formno', $studentFormNo)->value('id');
        return $studentId ?? null;
    }

    // public function getOldStudents()
    // {
    //     try {
    //         $oldStudents = OldStudentsRegister::all();

    //         if ($oldStudents->isEmpty()) {
    //             throw new Exception("No Old Students Found");
    //         }

    //         $groupedStudents = $oldStudents->groupBy('formno');

    //         $StudentDataSet = [];

    //         foreach ($groupedStudents as $formno => $students) {
    //             $firstStudent = $students->first();

    //             $actualStudentId = $this->getActualStudentId($firstStudent->formno);

    //             if (is_null($actualStudentId)) {
    //                 throw new Exception("Invalid");
    //             }
    //             $studentData = [
    //                 'registrationNumber' => $formno,
    //                 'students' => $students->map(function ($student) use ($actualStudentId) {
    //                     return [
    //                         'id' => $student->id,
    //                         'mainStudentId' =>  $actualStudentId,
    //                         'registrationNumber' => $student->formno,
    //                         'roomNumber' => $student->roomno,
    //                         'roomRent' => $this->getRoomRent($student->roomno),
    //                         'fees' => $this->getAmount($actualStudentId),
    //                         'seatNumber' => $student->seetno,
    //                         'name' => $student->candidatename,
    //                         'mobileNumber' => $student->phone,
    //                         'registrationDate' => $student->date,
    //                         'bithDate' => $student->dob,
    //                         'aadharNumber' => $student->valid,
    //                         'email' => $student->email,
    //                         'bloodGroup' => $student->blood,
    //                         'helthIssue' => $student->healthissue,
    //                         'healthDescription' => $student->health_description,
    //                         'stayDuration' => $student->stydrt,
    //                         'vehicalNumber' => $student->vehicalno,
    //                         'vehicleDescription' => $student->vehicledescription,
    //                         'courseName' => $student->coursename,
    //                         'courseDescription' => $student->coursedetails,
    //                         'jobdDescription' => $student->jobdetails,
    //                         'instituteCompany' => $student->institute,
    //                         'companyName' => $student->companyname,
    //                         'instituteName' => $student->institutename,
    //                         'fatherName' => $student->fathername,
    //                         'fatherOccupation' => $student->fatheroocc,
    //                         'motherName' => $student->mothername,
    //                         'motherOccupation' => $student->motheroocc,
    //                         'parentMobile1' => $student->tel,
    //                         'parentMobile2' => $student->mobilenumber,
    //                         'parentEmail' => $student->pemail,
    //                         'parentAddress' => $student->fullAddress,
    //                         'state' => $student->state,
    //                         'pincode' => $student->pinno,
    //                         'guardianName' => $student->guardianname,
    //                         'guardianMobileNumber' => $student->guardiannumber,
    //                         'guardianAddress' => $student->guardianaddress,
    //                         'CandidateImage' => $student->uploadimage ? asset('assets/uploads/' . $student->uploadimage) : null,
    //                         'aadharFrount' => $student->uploadaadharfront ? asset('assets/uploads/' . $student->uploadaadharfront) : null,
    //                         'AadharBack' => $student->uploadaadharback ? asset('assets/uploads/' . $student->uploadaadharback) : null,
    //                         'candidateSignature' => $student->candidatesign ? asset('assets/uploads/' . $student->candidatesign) : null,
    //                         'ParentSignature' => $student->Parentsign ? asset('assets/uploads/' . $student->Parentsign) : null,
    //                         'ChangeRoomStatus' => $student->ChangeRoomStatus,
    //                         'leaveDate' => $student->leave_date,
    //                     ];
    //                 })->toArray(),
    //             ];
    //             $StudentDataSet[] = $studentData;
    //         }
    //         return response()->json([
    //             'status' => true,
    //             'data' => $StudentDataSet,
    //         ], 200);
    //     } catch (Exception $e) {
    //         return response()->json([
    //             'status' => false,
    //             'error' => $e->getMessage(),
    //         ], 500);
    //     }
    // }
    public function getOldStudents(Request $request)
    {
        try {
            // Get the number of items per page from the query parameter or default to 10
            $perPage = $request->query('per_page', 10);

            // Fetch old students with pagination
            $oldStudents = OldStudentsRegister::orderBy('leave_date', 'desc')->paginate($perPage);

            if ($oldStudents->isEmpty()) {
                throw new Exception("No Old Students Found");
            }

            $groupedStudents = $oldStudents->groupBy('formno');

            $StudentDataSet = [];

            foreach ($groupedStudents as $formno => $students) {
                $firstStudent = $students->first();

                $actualStudentId = $this->getActualStudentId($firstStudent->formno);

                if (is_null($actualStudentId)) {
                    throw new Exception("Invalid");
                }

                $studentData = [
                    'registrationNumber' => $formno,
                    'students' => $students->map(function ($student) use ($actualStudentId) {
                        return [
                            'id' => $student->id,
                            'mainStudentId' =>  $actualStudentId,
                            'registrationNumber' => $student->formno,
                            'roomNumber' => $student->roomno,
                            'roomRent' => $this->getRoomRent($student->roomno),
                            'fees' => $this->getAmount($actualStudentId),
                            'seatNumber' => $student->seetno,
                            'name' => $student->candidatename,
                            'mobileNumber' => $student->phone,
                            'registrationDate' => $student->date,
                            'bithDate' => $student->dob,
                            'aadharNumber' => $student->valid,
                            'email' => $student->email,
                            'bloodGroup' => $student->blood,
                            'helthIssue' => $student->healthissue,
                            'healthDescription' => $student->health_description,
                            'stayDuration' => $student->stydrt,
                            'vehicalNumber' => $student->vehicalno,
                            'vehicleDescription' => $student->vehicledescription,
                            'courseName' => $student->coursename,
                            'courseDescription' => $student->coursedetails,
                            'jobdDescription' => $student->jobdetails,
                            'instituteCompany' => $student->institute,
                            'companyName' => $student->companyname,
                            'instituteName' => $student->institutename,
                            'fatherName' => $student->fathername,
                            'fatherOccupation' => $student->fatheroocc,
                            'motherName' => $student->mothername,
                            'motherOccupation' => $student->motheroocc,
                            'parentMobile1' => $student->tel,
                            'parentMobile2' => $student->mobilenumber,
                            'parentEmail' => $student->pemail,
                            'parentAddress' => $student->fullAddress,
                            'state' => $student->state,
                            'pincode' => $student->pinno,
                            'guardianName' => $student->guardianname,
                            'guardianMobileNumber' => $student->guardiannumber,
                            'guardianAddress' => $student->guardianaddress,
                            'CandidateImage' => $student->uploadimage ? asset('assets/uploads/' . $student->uploadimage) : null,
                            'aadharFrount' => $student->uploadaadharfront ? asset('assets/uploads/' . $student->uploadaadharfront) : null,
                            'AadharBack' => $student->uploadaadharback ? asset('assets/uploads/' . $student->uploadaadharback) : null,
                            'candidateSignature' => $student->candidatesign ? asset('assets/uploads/' . $student->candidatesign) : null,
                            'ParentSignature' => $student->Parentsign ? asset('assets/uploads/' . $student->Parentsign) : null,
                            'ChangeRoomStatus' => $student->ChangeRoomStatus,
                            'leaveDate' => $student->leave_date,
                        ];
                    })->toArray(),
                ];
                $StudentDataSet[] = $studentData;
            }

            return response()->json([
                'status' => true,
                'data' => $StudentDataSet,
                'pagination' => [
                    'current_page' => $oldStudents->currentPage(),
                    'last_page' => $oldStudents->lastPage(),
                    'per_page' => $oldStudents->perPage(),
                    'total' => $oldStudents->total(),
                ],
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }
    public function studentDetails($id)
    {
        try {
            $studentDetails = StudentRegister::findOrFail($id);
            $businessDetails = Businessregister::first();
            $hostelName = $businessDetails->businessname;
            $hostelId = $businessDetails->license_no;
            $studentDetails['hostelName'] = $hostelName;
            $studentDetails['hostelId'] = $hostelId;
            return response()->json([
                'status' => true,
                'message' => 'Student details fetched',
                'data' => $studentDetails,
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'message' => 'Student not found',
            ], 404);
        } catch (\Exception $e) { // Use backslash for global namespace
            return response()->json([
                'status' => false,
                'message' => 'Server Error: ' . $e->getMessage(),
            ], 500);
        }
    }
}
