<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use App\Models\ApiModels\StudentRegister;
use Illuminate\Support\Carbon;
use UnexpectedValueException;
use Exception;
use Illuminate\Validation\ValidationException;
use App\Models\ApiModels\Sales;
use App\Models\ApiModels\RecieveAmount;
use App\Models\ApiModels\CustomeSales;
use App\Models\ApiModels\ExpensesItem;
use App\Models\ApiModels\RoomSeats;
use App\Models\ApiModels\BusinessDocument;
use App\Models\Businessregister;
use DB;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use InvalidArgumentException;
use Illuminate\Support\Collection;
use App\Models\StudentElecticityRecords;
use App\Models\ApiModels\SaleVerification;
use Illuminate\Support\Facades\Mail;




class SaleController extends controller
{

    private const INVALID_DATA_ERROR_MESSAGE = "Invalid data. Sales form data is not set.";

    /**
     * Check if the invoice number already exists in the sales records.
     *
     * @param string $invoiceNumber The invoice number to check.
     * @return bool
     */
    public function getCandidateNames($roomNo)
    {
        try {
            // Get candidate names
            $students = StudentRegister::where('RoomNo', $roomNo)
                ->where('status', 1)
                ->get();

            if ($students->isEmpty()) {
                return response()->json([
                    'status' => true,
                    'data' => [],
                    'message' => 'Students not Found',
                ], 200);
            }

            $data = $students->map(function ($student) {
                return [
                    'id' => $student->id,
                    'name' => $student->candidatename,
                ];
            });

            return response()->json([
                'status' => true,
                'data' => $data,
            ], 200);
        } catch (UnexpectedValueException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    private function changeDate($regDate)
    {
        $regDate = Carbon::parse($regDate);
        $regDate->setDate(now()->year, now()->month, $regDate->format('d'));
        // Format the updated date as "YYYY-MM-DD"
        $updatedDate = $regDate->format('Y-m-d');
        return $updatedDate;
    }


    public function getCandidateDetails($studentId)
    {
        try {
            $student = StudentRegister::findOrFail($studentId);
            return response()->json([
                'status' => true,
                'data' => [
                    'registrationDate' => $student->join_date ? $this->changeDate($student->join_date) : $this->changeDate($student->date),
                    'mobileNumber' => $student->phone,
                    'address' => $student->fullAddress . ', ' . $student->state . ', ' . $student->pinno,
                ],
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => 'Student not found.',
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function getSalesItems()
    {
        try {
            $items = [];

            // Default items
            $defaultItems = [
                ['itemId' => 0, 'itemName' => 'Room Rent', 'itemCode' => "0"],
                ['itemId' => "e01", 'itemName' => 'Electricity Bill', 'itemCode' => "e01"],
            ];

            $salesItems = ExpensesItem::all();
            // Add default items
            $items = array_merge($items, $defaultItems);

            // Add remaining items from the salesItems to this array
            foreach ($salesItems as $item) {
                $items[] = [
                    'itemId' => $item->id,
                    "itemName" => $item->itemname,
                    'itemCode' => $item->itemcode,
                ];
            }

            return response()->json([
                'status' => true,
                'data' => $items
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function getLatestInvoiceNo()
    {
        try {
            // Get the latest invoice number from the Sales table
            $latestInvoiceNoSales = Sales::select('invoiceno')
                ->orderByDesc('invoiceno')
                ->first();

            // Get the latest invoice number from the CustomeSales table
            $latestInvoiceNoCustomeSales = CustomeSales::select('invoiceno')
                ->orderByDesc('invoiceno')
                ->first();

            // Merge the results and get the maximum invoice number
            $latestInvoiceNo = max(
                optional($latestInvoiceNoSales)->invoiceno,
                optional($latestInvoiceNoCustomeSales)->invoiceno
            );

            return response()->json([
                'status' => true,
                'data' => [
                    'latestInvoiceNo' => $latestInvoiceNo ? $latestInvoiceNo + 1 : 1,
                ],
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ]);
        }
    }


    public function getSalesListByStudentId($studentId)
    {
        try {
            $billDetails = [];
            $allBillsOfStudents = Sales::where('studentid', $studentId)->get();

            if ($allBillsOfStudents->isEmpty()) {
                throw new Exception("No Bills Found For This Student");
            }

            foreach ($allBillsOfStudents as $sale) {
                $billDetails[] = [
                    'invoiceNo' => $sale->invoiceno,
                    'pandingAmount' => $sale->remaining_amount,
                ];
            }

            return response()->json([
                'status' => true,
                'data' => $billDetails
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ]);
        }
    }


    public function getItemDetails(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                'itemCode' => 'required',
                'roomNumber' => 'nullable',
                'electricityDate' => 'nullable|date',
                'studentId' => 'nullable',
                'itemId' => 'nullable'
            ]);
            if ($validator->fails()) {
                throw new ValidationException($validator);
            }
            $responseData = [];
            if ($request->itemCode == "0") {
                // check room is given or not 
                if (!$request->has('roomNumber')) {
                    throw new UnexpectedValueException("Room Number is required for the item code {$request->itemCode}");
                }
                // get the room rent for the selected room
                $selectedRoom = RoomSeats::select('rent')->where("roomno", $request->roomNumber)->first();

                if (empty($selectedRoom)) {
                    throw new UnexpectedValueException("Room number {roomNumber} is not valid");
                }
                // genrate response for the selected room 
                $responseData = [
                    'itemCode' => $request->itemCode,
                    'rate' => $selectedRoom->rent,
                    'tax' => 0,
                    'amount' => $selectedRoom->rent,
                ];
            } elseif ($request->itemCode == 'e01') {
                // check the ElectricityDate is given or not
                if (!$request->has('electricityDate')) {
                    throw new UnexpectedValueException("Electricty Date is Required for the item code {$request->itemCode}");
                }
            } else {
                //check item id is given or not 
                if (!$request->has('itemId')) {
                    throw new UnexpectedValueException("Item Id is required for the item code {$request->itemCode}");
                }
                $items = ExpensesItem::findOrFail($request->itemId);
                $responseData = [
                    'itemCode' => $items['itemcode'],
                    'rate' => $items['rate'],
                    'tax' => $items['tax'],
                    'amount' => $items['amount']
                ];
            }

            return response()->json([
                'status' => true,
                'data' => $responseData,
            ], 200);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->errors()
            ], 422);
        } catch (UnexpectedValueException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 422);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    private function calculateColorAndAmount($totalSalesData, $unusedAmountData)
    {
        $color = 'black';
        $totalAmountShow = abs($totalSalesData - $unusedAmountData);

        if ($totalAmountShow > 0) {
            $color = $totalSalesData >= $unusedAmountData ? 'green' : 'orange';
        }

        return [
            'color' => $color,
            'amount' => number_format($totalAmountShow, 2),
        ];
    }

    public function salesDetails()
    {
        try {
            $salesData = [];
            $receiveAmount = Sales::select(DB::raw('SUM(remaining_amount) as receiveAmount'), DB::raw('SUM(unused) as unusedAmount'))
                ->get();

            $totalReceiveAmount = 0;
            $totalUnusedAmount = 0;

            foreach ($receiveAmount as $sale) {
                $totalReceiveAmount = $sale->receiveAmount;
                $totalUnusedAmount = $sale->unusedAmount;
            }

            if ($totalUnusedAmount > 0) {
                $totalReceiveAmount = max(0, $totalReceiveAmount - $totalUnusedAmount);
            }

            $currentMonth = Carbon::now()->month;
            $currentYear = Carbon::now()->year;
            $currentMonthTotal = Sales::whereMonth('invoicedate', $currentMonth)
                ->whereYear('invoicedate', $currentYear)
                ->select(DB::raw('SUM(total) as saletotal'))
                ->first();

            $totalAmount = $currentMonthTotal ? $currentMonthTotal->saletotal : 0;

            $salesTableData = Sales::join('studentregisters', 'sales.studentid', '=', 'studentregisters.id')
                ->select(
                    'studentregisters.id as studentid',
                    'studentregisters.candidatename',
                    'studentregisters.phone',
                    DB::raw('MAX(sales.tpdate) as tpdate'),
                    DB::raw('MAX(sales.invoicedate) as invoicedate'),
                    DB::raw('SUM(sales.remaining_amount) as totalAmount'),
                    DB::raw('SUM(sales.unused) as unusedAmount')
                )
                ->groupBy('studentregisters.id', 'studentregisters.candidatename')
                ->orderBy(DB::raw('COALESCE(MAX(sales.tpdate), MAX(sales.invoicedate))'), 'desc')
                ->get();

            if (!empty($salesTableData)) {
                foreach ($salesTableData as $row) {
                    $colorAndAmount = $this->calculateColorAndAmount($row->totalAmount, $row->unusedAmount);
                    $color = $colorAndAmount['color'];
                    $totalAmountShow = $colorAndAmount['amount'];

                    $salesData[] = [
                        'studentId' => $row->studentid,
                        'studentName' => $row->candidatename,
                        'studentMobile' => $row->phone,
                        'date' => Carbon::parse($row->tpdate)->format('d-M-Y'),
                        'amount' => $totalAmountShow,
                        'color' => $color,
                    ];
                }
            }

            return response()->json([
                'status' => true,
                'data' => [
                    'totalReceiveAmount' => number_format($totalReceiveAmount, 2),
                    'currentMonthSalesTotalAmount' => [
                        "month" => Carbon::now()->format("M"),
                        "Amount" => number_format($totalAmount, 2)
                    ],
                    'tableData' => $salesData
                ],
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'message' => $e->getMessage(),
            ], 500);
        }
    }

    public function customeSalesDetails()
    {
        try {
            $customSalesData = [];
            $customReceiveAmount = CustomeSales::select(DB::raw('SUM(remaining_amount) as receiveAmount'), DB::raw('SUM(unused) as unusedAmount'))
                ->get();

            $totalReceiveAmount = 0;
            $totalUnusedAmount = 0;

            foreach ($customReceiveAmount as $sale) {
                $totalReceiveAmount = $sale->receiveAmount;
                $totalUnusedAmount = $sale->unusedAmount;
            }

            if ($totalUnusedAmount > 0) {
                $totalReceiveAmount = max(0, $totalReceiveAmount - $totalUnusedAmount);
            }

            $currentMonth = Carbon::now()->month;
            $currentMonthTotal = CustomeSales::whereMonth('invoicedate', $currentMonth)
                ->select(DB::raw('SUM(total) as saletotal'))
                ->first();

            $totalAmount = $currentMonthTotal ? $currentMonthTotal->saletotal : 0;

            $salesTableData = CustomeSales::select('name', 'mobile',)
                ->selectRaw('MAX(id) as id')
                ->selectRaw('MAX(invoicedate) as latest_invoice_date')
                ->selectRaw('SUM(receive_amount) as receive_amount')
                ->selectRaw('SUM(remaining_amount) as remaining_amount')
                ->selectRaw('SUM(total) as total_amount')
                ->selectRaw('SUM(unused) as total_unused_amount')
                ->groupBy('mobile', 'name')
                ->orderByDesc('latest_invoice_date')
                ->get();
            if (!empty($salesTableData)) {
                foreach ($salesTableData as $row) {
                    $colorAndAmount = $this->calculateColorAndAmount($row->remaining_amount, $row->total_unused_amount);
                    $color = $colorAndAmount['color'];
                    $totalAmountShow = $colorAndAmount['amount'];

                    $customSalesData[] = [
                        'studentId' => $row->id,
                        'studentName' => $row->name,
                        'studentMobile' => $row->mobile,
                        'date' => Carbon::parse($row->latest_invoice_date)->format('d-M-Y'),
                        'amount' => $totalAmountShow,
                        'color' => $color,
                    ];
                }
            }

            return response()->json([
                'status' => true,
                'data' => [
                    'totalReceiveAmount' => number_format($totalReceiveAmount, 2),
                    'currentMonthSalesTotalAmount' => [
                        "month" => Carbon::now()->format("M"),
                        "Amount" => number_format($totalAmount, 2)
                    ],
                    'tableData' => $customSalesData
                ],
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'message' => $e->getMessage(),
            ], 500);
        }
    }

    private function prepareSalesItemList($sales)
    {
        $salesItems = [];
        $itemIds = explode(",", $sales->item);
        $itemRates = explode(",", $sales->rate);
        $itemQuantities = explode(",", $sales->quantity);
        $itemTax = explode(",", $sales->tax);
        $itemDiscount = explode(",", $sales->discount);
        $itemAmount = explode(",", $sales->amount);
        foreach ($itemIds as $key => $item) {
            if ($item === "0") {
                $itemName = "Room Rent";
            } elseif ($item === "e01") {
                $itemName = 'Electricity Bill';
            } else {
                $itemName = ExpensesItem::where('id', $item)->value('itemname');
            }
            $salesItems[] = [
                'itemId' => $item,
                'itemName' => $itemName,
                'itemRate' => $itemRates[$key],
                'itemQuantity' => $itemQuantities[$key],
                'itemTax' => $itemTax[$key],
                'itemDiscount' => $itemDiscount[$key],
                'itemAmount' => $itemAmount[$key],
            ];
        }

        return $salesItems;
    }

    private function prepareReceivedAmountList($receivedAmounts)
    {
        return collect($receivedAmounts)->map(function ($receiveAmount) {
            return [
                "id" => $receiveAmount->id,
                'receiveDate' => $receiveAmount->receiveDate,
                'invoiceNo' => $receiveAmount->invoiceNo,
                'unusedAmount' => $receiveAmount->unused,
                'receiveAmount' => $receiveAmount->receiveAmount,
                'paymentMode' => $receiveAmount->paymentMode,
            ];
        })->all();
    }


    public function studentSalesDetails($studentId)
    {
        try {
            $salesData = [];
            $totalRemainingAmount = 0;
            $totalUnusedAmount = 0;
            // Get Student Sales Data
            $salesRecords = Sales::with('receiveAmounts')->where('studentid', $studentId)
                ->orderBy("invoiceno", "desc")
                ->get();
            // get Student Details
            $studentDetails = StudentRegister::findOrFail($studentId);

            if ($salesRecords->isEmpty()) {
                throw new UnexpectedValueException("No Records Found for this student ID {$studentId}");
            }

            foreach ($salesRecords as $sales) {
                $totalRemainingAmount += $sales->remaining_amount;
                $totalUnusedAmount += $sales->unused;

                $salesItems = $this->prepareSalesItemList($sales);
                $receiveData = $this->prepareReceivedAmountList($sales->receiveAmounts);
                $salesData[] = [
                    'studentRoomNumber' => $sales->roomno,
                    'invoiceNumber' => $sales->invoiceno,
                    'invoiceDate' => $sales->invoicedate,
                    'totalAmount' => $sales->total,
                    'receivedAmount' => $sales->receive_amount,
                    'remainingAmount' => $sales->remaining_amount,
                    'unusedAmount' => $sales->unused,
                    'receivedDate' => $receiveData,
                    'items' => $salesItems,
                ];
            }

            return response()->json([
                'status' => true,
                'data' => [
                    "studentDetails" => [
                        'studentId' => $studentId,
                        'studentName' => $studentDetails->candidatename,
                        'mobileNumber' => $studentDetails->phone,
                        'RegistrationDate' =>  $studentDetails->join_date ? $this->changeDate($studentDetails->join_date) : $this->changeDate($studentDetails->date),
                        'address' => $studentDetails->fullAddress . ", " . $studentDetails->state . ', ' . $studentDetails->pinno,
                        'TotalRemainingAmount' => $totalRemainingAmount,
                        'TotalUnusedAmount' => $totalUnusedAmount,
                    ],
                    'salesDetails' => $salesData,
                ],
            ]);
        } catch (UnexpectedValueException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function customStudentSalesDetails($studentName, $studentMobile)
    {
        try {
            // Getting student data from the custom sales table
            $customeSalesData = [];
            $totalRemainingAmount = 0;
            $totalUnusedAmount = 0;
            $customSales = CustomeSales::where('name', $studentName)
                ->where('mobile', $studentMobile)
                ->first();

            if (!$customSales) {
                return response()->json([
                    'status' => false,
                    'error' => 'Student not found',
                ]);
            }

            // Get all sales related to the student
            $salesDetails = CustomeSales::where('name', $studentName)
                ->where('mobile', $studentMobile)
                ->get();

            foreach ($salesDetails as $sales) {
                $totalRemainingAmount += $sales->remaining_amount;
                $totalUnusedAmount += $sales->unused;
                $salesItems = $this->prepareSalesItemList($sales);
                $customeSalesData[] = [
                    'studentRoomNumber' => $sales->roomno,
                    'invoiceNumber' => $sales->invoiceno,
                    'invoiceDate' => $sales->invoicedate,
                    'totalAmount' => $sales->total,
                    'receivedAmount' => $sales->receive_amount,
                    'remainingAmount' => $sales->remaining_amount,
                    'unusedAmount' => $sales->unused,
                    'receivedDate' => [],
                    'salesItem' => $salesItems,
                ];
            }


            // Continue processing and build the response
            return response()->json([
                'status' => true,
                'data' => [
                    'studentDetails' => [
                        'studentName' => $customSales->name,
                        'mobileNumber' => $customSales->mobile,
                        "address" => $customSales->address,
                        "TotalRemainingAmount" => $totalRemainingAmount,
                        "TotalUnusedAmount" => $totalUnusedAmount,
                    ],
                    'salesDetails' => $customeSalesData,
                ]
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ]);
        }
    }
    private function checkInvoiceExist($invoiceNumber, $type = ""): bool
    {
        try {
            if ($type == "" || $type == "saleUpdate") {
                $salesExist = Sales::where("invoiceno", $invoiceNumber)->first();
            } else {
                $salesExist = CustomeSales::where("invoiceno", $invoiceNumber)->first();
            }

            $type = ($type != "" && $type != "custom") ? "update" : '';
            return ($type == 'update') ? ($salesExist ? true : false) : !$salesExist;
        } catch (Exception $e) {
            return false;
        }
    }

    /**
     * Check if a student with the given ID exists.
     *
     * @param int $studentId The ID of the student.
     * @return bool
     */
    private function checkStudentExist($studentId): bool
    {
        try {
            StudentRegister::where('id', $studentId)->firstOrFail();
            return true;
        } catch (ModelNotFoundException $e) {
            return false;
        } catch (Exception $e) {
            return false;
        }
    }

    /**
     * Insert payment information.
     *
     * @param string $receiveDate The date of the payment.
     * @param int $candidateID The ID of the candidate.
     * @param string $invoiceNo The invoice number.
     * @param float $pendingAmount The pending amount.
     * @param float $receiveAmount The received amount.
     * @param string $paymentMode The payment mode.
     * @return \Illuminate\Http\JsonResponse
     */
    private function insertPaymentIn($receiveDate, $candidateID, $invoiceNo, $pendingAmount, $receiveAmount, $paymentMode, $remark = '', $returnRequred = true)
    {
        try {
            $receiveAmountModel = new RecieveAmount();
            $receiveAmountModel->receiveDate = $receiveDate;
            $receiveAmountModel->candidateID = $candidateID;
            $receiveAmountModel->invoiceNo = $invoiceNo;

            if ($pendingAmount >= 0) {
                $receiveAmountModel->pandingAmount = $pendingAmount;
            } else {
                $receiveAmountModel->unused = abs($pendingAmount);
                $receiveAmountModel->pandingAmount = 0;
            }

            $receiveAmountModel->receiveAmount = $receiveAmount;
            $receiveAmountModel->paymentMode = $paymentMode;
            $receiveAmountModel->remark = $remark;

            if ($returnRequred) {
                if ($receiveAmountModel->save()) {
                    return response()->json(['message' => 'Data saved successfully']);
                } else {
                    return response()->json(['message' => 'Failed to save data'], 500);
                }
            } else {
                if (!$receiveAmountModel->save()) {
                    throw new Exception('Failed to save payment data');
                }
            }
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => 'An error occurred while processing your request',
            ], 500);
        }
    }

    /**
     * Store sales information.
     *
     * @param \Illuminate\Http\Request $request The incoming HTTP request.
     * @return \Illuminate\Http\JsonResponse
     */

    private function deletePaymentIn($invoiceNo, $studentId): bool
    {
        try {
            $deletedRows = RecieveAmount::where('invoiceNo', $invoiceNo)
                ->where('candidateID', $studentId)
                ->delete();
            return true;
        } catch (Exception $e) {
            return false;
        }
    }

    private function checkAndUpdateElectricityBill($itemList, $candidateId, $invoiceNo)
    {
        foreach ($itemList as $key => $value) {
            if ($value['itemId'] == "e01") {
                $electricityBillId = $value['electricityId'];
                $studentElectricityRecords = StudentElecticityRecords::where("id", $electricityBillId)
                    ->where("userId", $candidateId)
                    ->where("status", 0)
                    ->first();

                if ($studentElectricityRecords) {
                    $studentElectricityRecords->status = 1;
                    $studentElectricityRecords->invoiceno = $invoiceNo;
                    $studentElectricityRecords->save(); // Use save() since we've manually set the properties
                    return true; // Return the updated record or a success message
                } else {
                    throw new ModelNotFoundException("No electricity records found for ID: {$electricityBillId}");
                }
            }
        }
        return true; // if no item which is related to electricity
    }

    public function storeSales(Request $request)
    {
        try {
            DB::beginTransaction(); // Start a new database transaction
            $salesFormDate = $request->input('salesFormData');
            $salesItems = $request->input('salesItemsList');

            if (!isset($salesFormDate)) {
                throw new UnexpectedValueException(self::INVALID_DATA_ERROR_MESSAGE);
            }

            if (empty($salesFormDate["invoiceNo"]) || !is_numeric($salesFormDate["invoiceNo"])  || !$this->checkInvoiceExist($salesFormDate["invoiceNo"])) {
                throw new UnexpectedValueException(!is_numeric($salesFormDate["invoiceNo"]) ? "Invoice number must be integer" :  'Invoice no is not valid.');
            }

            if (!$this->checkStudentExist($salesFormDate["studentId"])) {
                throw new ModelNotFoundException("Student not found for this ID: {$salesFormDate['studentId']}");
            }

            if (count($salesItems) == 0) {
                throw new InvalidArgumentException("Item List can't be empty.");
            }
            $itemIds = [];
            $itemRates = [];
            $itemQuantities = [];
            $itemTax = [];
            $itemDiscount = [];
            $itemAmount = [];
            foreach ($salesItems as $item) {
                $itemIds[] = $item['itemId'] ?? throw new InvalidArgumentException("In the Sale Item List, item id not be this: {$item['itemId']}");
                $itemRates[] = is_numeric($item['itemRate']) ? $item['itemRate'] : throw new InvalidArgumentException("In the Sale Item List, item rate not be this: {$item['itemRate']}");
                $itemQuantities[] = is_numeric($item['itemQuantity']) ? $item['itemQuantity'] : throw new InvalidArgumentException("In the Sale Item List, item quantity not be this: {$item['itemQuantity']}");
                $itemTax[] = is_numeric($item['itemTax']) ? $item['itemTax'] : throw new InvalidArgumentException("In the Sale Item List, item tax not uploaded properly be this: {$item['itemTax']}");
                $itemDiscount[] = is_numeric($item['itemDiscount']) ? $item['itemDiscount'] : throw new InvalidArgumentException("In the Sale Item List, item discount not be this: {$item['itemDiscount']}");
                $itemAmount[] = is_numeric($item['itemAmount']) ? $item['itemAmount'] : throw new InvalidArgumentException("In the Sale Item List, item amount not be this: {$item['itemAmount']}");
            }
            $receiveAmount = $salesFormDate['receivedAmount'];
            $totalAmount = $salesFormDate['totalAmount'];
            $remaningAmount = (float) $totalAmount - (float)$receiveAmount;

            $sale = new Sales();
            $sale->StudentId = (int) $salesFormDate['studentId'];
            $sale->roomno = (int) $salesFormDate['roomNumber'] ?? throw new UnexpectedValueException('Student Room Number is required');
            $sale->invoiceno = $salesFormDate['invoiceNo'];
            $sale->invoicedate = $salesFormDate['saleDate'] ?? throw new UnexpectedValueException("Sale Date is required");
            $sale->tpdate = $salesFormDate['saleDate'];
            $sale->item = implode(',', $itemIds);
            $sale->quantity = implode(',', $itemQuantities);
            $sale->rate = implode(',', $itemRates);
            $sale->tax = implode(',', $itemTax);
            $sale->amount = implode(',', $itemAmount);
            $sale->discount = implode(',', $itemDiscount);
            $sale->receive_amount = $receiveAmount;
            $sale->total = $totalAmount;

            if ($remaningAmount >= 0) {
                $sale->payment_status = 1;
                $sale->remaining_amount = (string) $remaningAmount;
                $receiveAmount > 0 ? $this->insertPaymentIn(
                    $salesFormDate['saleDate'],
                    $salesFormDate['studentId'],
                    $salesFormDate['invoiceNo'],
                    $remaningAmount,
                    $receiveAmount,
                    $salesFormDate['paymentMode']
                ) : null;
            } elseif ($remaningAmount < 0) {
                $sale->payment_status = 0;
                $sale->unused = abs($remaningAmount);
                $this->insertPaymentIn(
                    $salesFormDate['saleDate'],
                    $salesFormDate['studentId'],
                    $salesFormDate['invoiceNo'],
                    $remaningAmount,
                    $receiveAmount,
                    $salesFormDate['paymentMode']
                );
            }
            // Save the sales information
            $sale->save();
            // if have electricity bill data then make it
            $electricityRecrods = $this->checkAndUpdateElectricityBill($salesItems, $salesFormDate['studentId'], $salesFormDate['invoiceNo']);
            // Check if $electricityRecrods is true or contains the expected data
            if ($electricityRecrods !== true) {
                // If $electricityRecrods is not true, roll back the transaction
                DB::rollBack();
                return response()->json([
                    'status' => false,
                    'error' => 'Electricity record update failed',
                ], 422);
            }

            // If everything is fine, commit the transaction
            DB::commit();

            return response()->json([
                'status' => true,
                'message' => 'Sales data saved successfully',
                'invoiceNumber' => $sale->invoiceno,
            ], 200);
        } catch (InvalidArgumentException | UnexpectedValueException $e) {
            DB::rollBack();
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 422);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function storeCustomSales(Request $request)
    {
        try {
            $salesFormDate = $request->input('salesFormData');
            $salesItems = $request->input('salesItemsList');
            if (!isset($salesFormDate)) {
                throw new UnexpectedValueException(self::INVALID_DATA_ERROR_MESSAGE);
            }
            if (empty($salesFormDate["invoiceNo"]) || !is_numeric($salesFormDate["invoiceNo"])  || !$this->checkInvoiceExist($salesFormDate["invoiceNo"], $type = "custom")) {
                throw new UnexpectedValueException(!is_numeric($salesFormDate["invoiceNo"]) ? 'Invoice number must be integer' : "Invoice number is invalid");
            }
            if (count($salesItems) == 0) {
                throw new InvalidArgumentException("Item List can't be empty.");
            }
            $itemIds = [];
            $itemRates = [];
            $itemQuantities = [];
            $itemTax = [];
            $itemDiscount = [];
            $itemAmount = [];
            foreach ($salesItems as $item) {
                $itemIds[] = $item['itemId'] ?? throw new InvalidArgumentException("In the Sale Item List, item id not be this: {$item['itemId']}");
                $itemRates[] = is_numeric($item['itemRate']) ? $item['itemRate'] : throw new InvalidArgumentException("In the Sale Item List, item rate not be this: {$item['itemRate']}");
                $itemQuantities[] = is_numeric($item['itemQuantity']) ? $item['itemQuantity'] : throw new InvalidArgumentException("In the Sale Item List, item quantity not be this: {$item['itemQuantity']}");
                $itemTax[] = is_numeric($item['itemTax']) ? $item['itemTax'] : throw new InvalidArgumentException("In the Sale Item List, item tax not uploaded properly be this: {$item['itemTax']}");
                $itemDiscount[] = is_numeric($item['itemDiscount']) ? $item['itemDiscount'] : throw new InvalidArgumentException("In the Sale Item List, item discount not be this: {$item['itemDiscount']}");
                $itemAmount[] = is_numeric($item['itemAmount']) ? $item['itemAmount'] : throw new InvalidArgumentException("In the Sale Item List, item amount not be this: {$item['itemAmount']}");
            }
            $receiveAmount = $salesFormDate['receivedAmount'];
            $totalAmount = $salesFormDate['totalAmount'];
            $remaningAmount = $totalAmount - $receiveAmount;

            $sale = new CustomeSales();
            $sale->roomno = (int) $salesFormDate['roomNumber'] ?? throw new InvalidArgumentException('Room No is required');
            $sale->name = $salesFormDate['candidateName'] ?? throw new InvalidArgumentException('Candidate Name is required');
            $sale->mobile =  $salesFormDate['mobile'] ?? throw new InvalidArgumentException('Mobile Number is required');
            $sale->address =  $salesFormDate['address'] ?? throw new InvalidArgumentException('candidate Address is required');
            $sale->invoiceno =  $salesFormDate['invoiceNo'];
            $sale->invoicedate = $salesFormDate['saleDate'] ?? throw new InvalidArgumentException('Sale Date is required');
            $sale->tpdate = $salesFormDate['saleDate'] ?? throw new InvalidArgumentException('Sale Date is required');

            $sale->item = implode(',', $itemIds);
            $sale->quantity = implode(',', $itemQuantities);
            $sale->rate = implode(',', $itemRates);
            $sale->tax = implode(',', $itemTax);
            $sale->amount = implode(',', $itemAmount);
            $sale->discount = implode(',', $itemDiscount);
            $sale->receive_amount = $receiveAmount;
            $sale->total = $totalAmount;
            $sale->paymentMode = $salesFormDate['paymentMode'] ?? null;
            if ($remaningAmount >= 0) {
                $sale->payment_status = 1;
                $sale->remaining_amount = (string) $remaningAmount;
            } else if ($remaningAmount < 0) {
                $sale->payment_status = 0;
                $sale->remaining_amount = 0;
                $sale->unused = abs($remaningAmount);
            }
            $sale->save();
            return response()->json([
                'status' => true,
                'message' => 'custome sales saved successfully',
                'invoiceNumber' => $sale->invoiceno,
            ], 200);
        } catch (InvalidArgumentException | UnexpectedValueException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 422);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function updateStoreSales(Request $request)
    {
        try {
            DB::beginTransaction(); // Start a new database transaction

            $salesFormDate = $request->input('salesFormData');
            $salesItems = $request->input('salesItemsList');

            if (!isset($salesFormDate)) {
                throw new UnexpectedValueException(self::INVALID_DATA_ERROR_MESSAGE);
            }


            if (empty($salesFormDate["invoiceNo"]) || !is_numeric($salesFormDate["invoiceNo"])  || !$this->checkInvoiceExist($salesFormDate["invoiceNo"], $type = "saleUpdate")) {
                throw new UnexpectedValueException(!is_numeric($salesFormDate["invoiceNo"]) ? "Invoice number must be integer" :  'Invoice no is not valid.');
            }

            if (!$this->checkStudentExist($salesFormDate["studentId"])) {
                throw new ModelNotFoundException("Student not found for this ID: {$salesFormDate['studentId']}");
            }

            if (count($salesItems) == 0) {
                throw new InvalidArgumentException("Item List can't be empty.");
            }


            $itemIds = [];
            $itemRates = [];
            $itemQuantities = [];
            $itemTax = [];
            $itemDiscount = [];
            $itemAmount = [];
            foreach ($salesItems as $item) {
                $itemIds[] = $item['itemId'] ?? throw new InvalidArgumentException("In the Sale Item List, item id not be this: {$item['itemId']}");
                $itemRates[] = is_numeric($item['itemRate']) ? $item['itemRate'] : throw new InvalidArgumentException("In the Sale Item List, item rate not be this: {$item['itemRate']}");
                $itemQuantities[] = is_numeric($item['itemQuantity']) ? $item['itemQuantity'] : throw new InvalidArgumentException("In the Sale Item List, item quantity not be this: {$item['itemQuantity']}");
                $itemTax[] = is_numeric($item['itemTax']) ? $item['itemTax'] : throw new InvalidArgumentException("In the Sale Item List, item tax not uploaded properly be this: {$item['itemTax']}");
                $itemDiscount[] = is_numeric($item['itemDiscount']) ? $item['itemDiscount'] : throw new InvalidArgumentException("In the Sale Item List, item discount not be this: {$item['itemDiscount']}");
                $itemAmount[] = is_numeric($item['itemAmount']) ? $item['itemAmount'] : throw new InvalidArgumentException("In the Sale Item List, item amount not be this: {$item['itemAmount']}");
            }
            $receiveAmount = $salesFormDate['receivedAmount'];
            $totalAmount = $salesFormDate['totalAmount'];
            $remaningAmount = $totalAmount - $receiveAmount;

            // delete payment In records before updating the sales records 

            $deletePaymentInRecords = $this->deletePaymentIn($salesFormDate['invoiceNo'], $salesFormDate['studentId']);
            $checkElectricityRecords = StudentElecticityRecords::where("invoiceno", $salesFormDate['invoiceNo'])->count();


            $deletePaymentInRecords ?? throw new Exception("Somthing want Wrong to delete Received amount records");

            $sale = Sales::where('invoiceno', $salesFormDate['invoiceNo'])->firstOrFail();
            $sale->StudentId = (int) $salesFormDate['studentId'];
            $sale->roomno = (int) $salesFormDate['roomNumber'] ?? throw new UnexpectedValueException('Student Room Number is required');
            $sale->invoiceno = $salesFormDate['invoiceNo'];
            $sale->invoicedate = $salesFormDate['saleDate'] ?? throw new UnexpectedValueException("Sale Date is required");
            $sale->tpdate = $salesFormDate['saleDate'];
            $sale->item = implode(',', $itemIds);
            $sale->quantity = implode(',', $itemQuantities);
            $sale->rate = implode(',', $itemRates);
            $sale->tax = implode(',', $itemTax);
            $sale->amount = implode(',', $itemAmount);
            $sale->discount = implode(',', $itemDiscount);
            $sale->receive_amount = $receiveAmount;
            $sale->total = $totalAmount;

            if ($remaningAmount >= 0) {
                $sale->payment_status = 1;
                $sale->remaining_amount = (string) $remaningAmount;
                $receiveAmount > 0 ? $this->insertPaymentIn(
                    $salesFormDate['saleDate'],
                    $salesFormDate['studentId'],
                    $salesFormDate['invoiceNo'],
                    $remaningAmount,
                    $receiveAmount,
                    $salesFormDate['paymentMode']
                ) : null;
            } elseif ($remaningAmount < 0) {
                $sale->payment_status = 0;
                $sale->unused = abs($remaningAmount);
                $this->insertPaymentIn(
                    $salesFormDate['saleDate'],
                    $salesFormDate['studentId'],
                    $salesFormDate['invoiceNo'],
                    $remaningAmount,
                    $receiveAmount,
                    $salesFormDate['paymentMode']
                );
            }
            // Save the sales information
            $sale->update();
            if ($checkElectricityRecords > 0) {
                $updatedElectricityRecords = StudentElecticityRecords::where("invoiceno", $salesFormDate['invoiceNo'])->update([
                    'status' => 0,
                    'invoiceno' => null,
                ]);
            }
            $electricityRecrods = $this->checkAndUpdateElectricityBill($salesItems, $salesFormDate['studentId'], $salesFormDate['invoiceNo']);
            // Check if $electricityRecrods is true or contains the expected data
            if ($electricityRecrods !== true) {
                // If $electricityRecrods is not true, roll back the transaction
                DB::rollBack();
                return response()->json([
                    'status' => false,
                    'error' => 'Electricity record update failed',
                ], 422);
            }

            // If everything is fine, commit the transaction
            DB::commit();
            return response()->json([
                'status' => true,
                'message' => 'Sales data update successfully',
            ], 200);
        } catch (InvalidArgumentException | UnexpectedValueException $e) {
            DB::rollBack();
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 422);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function updateCustomeSales(Request $request)
    {
        try {
            $salesFormDate = $request->input('salesFormData');
            $salesItems = $request->input('salesItemsList');
            if (!isset($salesFormDate)) {
                throw new UnexpectedValueException(self::INVALID_DATA_ERROR_MESSAGE);
            }
            if (empty($salesFormDate["invoiceNo"]) || !is_numeric($salesFormDate["invoiceNo"]) || !$this->checkInvoiceExist($salesFormDate["invoiceNo"], $type = "customUpdate")) {
                throw new UnexpectedValueException(!is_numeric($salesFormDate["invoiceNo"]) ? 'Invoice number must be integer' : 'Invoice no is not valid.');
            }
            if (count($salesItems) == 0) {
                throw new InvalidArgumentException("Item List can't be empty.");
            }
            $itemIds = [];
            $itemRates = [];
            $itemQuantities = [];
            $itemTax = [];
            $itemDiscount = [];
            $itemAmount = [];
            foreach ($salesItems as $item) {
                $itemIds[] = $item['itemId'] ?? throw new InvalidArgumentException("In the Sale Item List, item id not be this: {$item['itemId']}");
                $itemRates[] = is_numeric($item['itemRate']) ? $item['itemRate'] : throw new InvalidArgumentException("In the Sale Item List, item rate not be this: {$item['itemRate']}");
                $itemQuantities[] = is_numeric($item['itemQuantity']) ? $item['itemQuantity'] : throw new InvalidArgumentException("In the Sale Item List, item quantity not be this: {$item['itemQuantity']}");
                $itemTax[] = is_numeric($item['itemTax']) ? $item['itemTax'] : throw new InvalidArgumentException("In the Sale Item List, item tax not uploaded properly be this: {$item['itemTax']}");
                $itemDiscount[] = is_numeric($item['itemDiscount']) ? $item['itemDiscount'] : throw new InvalidArgumentException("In the Sale Item List, item discount not be this: {$item['itemDiscount']}");
                $itemAmount[] = is_numeric($item['itemAmount']) ? $item['itemAmount'] : throw new InvalidArgumentException("In the Sale Item List, item amount not be this: {$item['itemAmount']}");
            }
            $receiveAmount = $salesFormDate['receivedAmount'];
            $totalAmount = $salesFormDate['totalAmount'];
            $remaningAmount = $totalAmount - $receiveAmount;

            $sale = CustomeSales::where('invoiceno', $salesFormDate['invoiceNo'])->firstOrFail();
            $sale->roomno = (int) $salesFormDate['roomNumber'] ?? throw new InvalidArgumentException('Room No is required');
            $sale->name = $salesFormDate['candidateName'] ?? throw new InvalidArgumentException('Candidate Name is required');
            $sale->mobile =  $salesFormDate['mobile'] ?? throw new InvalidArgumentException('Mobile Number is required');
            $sale->address =  $salesFormDate['address'] ?? throw new InvalidArgumentException('candidate Address is required');
            $sale->invoiceno =  $salesFormDate['invoiceNo'];
            $sale->invoicedate = $salesFormDate['saleDate'] ?? throw new InvalidArgumentException('Sale Date is required');
            $sale->tpdate = $salesFormDate['saleDate'] ?? throw new InvalidArgumentException('Sale Date is required');

            $sale->item = implode(',', $itemIds);
            $sale->quantity = implode(',', $itemQuantities);
            $sale->rate = implode(',', $itemRates);
            $sale->tax = implode(',', $itemTax);
            $sale->amount = implode(',', $itemAmount);
            $sale->discount = implode(',', $itemDiscount);
            $sale->receive_amount = $receiveAmount;
            $sale->total = $totalAmount;
            $sale->paymentMode = $salesFormDate['paymentMode'] ?? null;
            if ($remaningAmount >= 0) {
                $sale->payment_status = 1;
                $sale->remaining_amount = (string) $remaningAmount;
            } else if ($remaningAmount < 0) {
                $sale->payment_status = 0;
                $sale->remaining_amount = 0;
                $sale->unused = abs($remaningAmount);
            }
            $sale->save();
            return response()->json([
                'status' => true,
                'message' => 'custome sales saved successfully'
            ], 200);
        } catch (InvalidArgumentException | UnexpectedValueException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 422);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function makePaymentInForSale(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                "invoiceNo" => 'required',
                "studentId" => 'required|numeric',
                "pandingAmount" => 'required|numeric',
                'receivedAmount' => 'required|numeric',
                'receivedDate' => 'required|date',
                'paymentMode' => 'required|string',
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            // Check if sale exists for the given invoice number
            $sales = $this->checkInvoiceExist($request->invoiceNo, 'saleUpdate');
            if (!$sales) {
                throw new UnexpectedValueException('Records not found for this sale');
            }

            // Update existing record in RecieveAmount
            RecieveAmount::create([
                'receiveDate' => $request->receivedDate,
                'candidateID' => $request->studentId,
                'invoiceNo' => $request->invoiceNo,
                'receiveAmount' => $request->receivedAmount,
                'paymentMode' => $request->paymentMode,
            ]);
            // calculate total received amount from received amount table and then according to candition update in sale table
            $this->calculateTotalReceivedAmountAndUpdateSale($request);

            return response()->json([
                'status' => true,
                'message' => 'Payment made successfully',
            ], 200);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->errors(),
            ], 422);
        } catch (UnexpectedValueException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => 'An unexpected error occurred.',
            ], 500);
        }
    }

    public function updatePaymentInForSale(Request $request, $payemntInId)
    {
        try {
            $validator = Validator::make($request->all(), [
                "invoiceNo" => 'required',
                "studentId" => 'required|numeric',
                'receivedAmount' => 'required|numeric',
                'receivedDate' => 'required|date',
                'paymentMode' => 'required|string',
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            // Check if sale exists for the given invoice number
            $sales = $this->checkInvoiceExist($request->invoiceNo, 'saleUpdate');
            if (!$sales) {
                throw new UnexpectedValueException('Records not found for this sale');
            }

            // Update existing record in RecieveAmount
            RecieveAmount::where('id', $payemntInId)
                ->update([
                    'receiveDate' => $request->receivedDate,
                    'candidateID' => $request->studentId,
                    'invoiceNo' => $request->invoiceNo,
                    'receiveAmount' => $request->receivedAmount,
                    'paymentMode' => $request->paymentMode,
                ]);

            // calculate total received amount from received amount table and then according to candition update in sale table
            $this->calculateTotalReceivedAmountAndUpdateSale($request);

            return response()->json([
                'status' => true,
                'message' => 'Payment updated successfully',
            ], 200);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->errors(),
            ], 422);
        } catch (UnexpectedValueException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => 'An unexpected error occurred.',
            ], 500);
        }
    }

    private function calculateTotalReceivedAmountAndUpdateSale($request)
    {
        try {
            $totalReceivedAmount = RecieveAmount::where('invoiceNo', $request->invoiceNo)
                ->sum('receiveAmount');

            // Update the main sales table
            $salesTable = Sales::where('invoiceno', $request->invoiceNo)->first();

            $totalSaleAmount = $salesTable->total;
            $salesUnusedAmount = max(0, $totalReceivedAmount - $totalSaleAmount);
            $salePandingAmount = max(0, $totalSaleAmount - $totalReceivedAmount);

            $salesTable->update([
                'unused' => $salesUnusedAmount,
                'receive_amount' => $totalReceivedAmount,
                'remaining_amount' => $salePandingAmount,
            ]);
        } catch (Exception $e) {
            throw new Exception("Error in calculating total received amount and updating sale: " . $e->getMessage());
        }
    }

    public function studentSalesAccountDetails($studentId)
    {
        try {
            $studentSalesDetails = Sales::where('studentid', $studentId)->first();
            if (empty($studentSalesDetails)) {
                return response()->json([
                    'status' => true,
                    'statusCode' => 200,
                    'message' => 'Sales Data Not Found',
                    'data' => [],
                ], 200);
            }


            $invoiceNo = []; //this is for totalDuesSales (which is have all the sales invoice no which is not paid)
            $lastBillDueRemainingAmount = Sales::where('studentid', $studentId)
                ->orderBy('id', 'desc')
                ->first();

            $totalDueSales = Sales::where('studentid', $studentId)->where('remaining_amount', '>', 0)->get();
            if (!empty($totalDueSales)) {
                foreach ($totalDueSales as $sales) {
                    $invoiceNo[] = $sales->invoiceno;
                }
            }
            $totalDueAmount = $totalDueSales->sum('remaining_amount');
            $salesItems = $this->prepareSalesItemList($studentSalesDetails);
            $totalAmount = $studentSalesDetails->total;
            $receiveAmount = $studentSalesDetails->receive_amount;
            if (round($receiveAmount) == round($totalAmount)) {
                $ispaid = true;
            } else {
                $ispaid = false;
            }
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => "Sales details  found successfully",
                'data' => [
                    'ispaid' => $ispaid,
                    'firstBillItem' => $salesItems,
                    'billDetails' => [
                        [
                            'name' => 'Current Due',
                            'amount' => $lastBillDueRemainingAmount->remaining_amount,
                            'invoiceNo' => $lastBillDueRemainingAmount->invoiceno
                        ],
                        [
                            'name' => 'Total Due',
                            'amount' => $totalDueAmount,
                            'invoiceNo' => $invoiceNo
                        ]
                    ]
                ]
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => $e->getMessage(),
            ], 500);
        }
    }

    public function deleteMainSales(Request $request, $invoiceNo)
    {
        try {
            // Start transaction
            DB::beginTransaction();
            // Get the sales record
            $sale = Sales::where('invoiceno', $invoiceNo)->firstOrFail();

            // Get its paymentIn records and delete them
            $receiveAmountRecords = RecieveAmount::where('invoiceNo', $invoiceNo)->get();
            if (!empty($receiveAmountRecords)) {
                foreach ($receiveAmountRecords as $record) {
                    $record->delete();
                }
            }
            // Now delete the sales record
            $sale->delete();

            // Commit the transaction
            DB::commit();

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => "Sales record and associated payments deleted successfully.",
            ], 200);
        } catch (ModelNotFoundException $e) {
            // Rollback the transaction if an exception occurs
            DB::rollBack();

            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => "Record not found for invoice no: $invoiceNo",
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            // Rollback the transaction if an exception occurs
            DB::rollBack();

            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => "Something went wrong!",
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function deleteCustomeSales($invoiceNo)
    {
        try {

            // Start a database transaction
            DB::beginTransaction();
            // Get sales data by invoice number

            $customeSale = CustomeSales::where('invoiceno', $invoiceNo)->firstOrFail();

            $customeSale->delete();

            DB::commit();

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Custome Sale is deleted successfully'
            ], 200);
        } catch (ModelNotFoundException $e) {
            // Rollback the transaction if an exception occurs
            DB::rollBack();

            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => "Record not found for invoice no: $invoiceNo",
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => "Somthing went wrong!",
                'error' => $e->getMessage()
            ], 500);
        }
    }

    protected function updateOtpForTheSale($sale, $otp)
    {
        //in which the particuler sale update the $otp of the sale
        $sale->otp = $otp;
        $sale->save();
    }

    public function shareOtpForSale(Request $request)
    {
        try {
            // Validate the request data
            $validation = Validator::make($request->all(), [
                'studentId' => 'required|integer',
                'saleType' => 'required|string|in:currentDue,totalDue,mainSale',
                'invoiceNo' => 'required',
                'amount' => 'required|numeric|min:1',
                'type' => 'required|string|in:online,cash',
            ]);

            if ($validation->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 402,
                    'message' => 'Validation Error',
                    'error' => $validation->errors(),
                ]);
            }

            // Retrieve business details
            $businessDetails = Businessregister::first();

            // Retrieve student details
            $student = Studentregister::findOrFail($request->studentId);

            // Generate OTP (for demonstration purposes, you might want to implement your own OTP generation logic)
            $otp = rand(1000, 9999); // Generate a random OTP

            switch ($request->saleType) {
                case 'currentDue':
                    $lastBillDueSaleRecord = Sales::where('studentid', $student->id)
                        ->where('invoiceno', $request->invoiceNo)
                        ->orderBy('id', 'desc')
                        ->first();
                    $this->updateOtpForTheSale($lastBillDueSaleRecord, $otp);
                    break;
                case 'totalDue':
                    $invoiceNos = explode(',', $request->invoiceNo);
                    $totalRemainingAmount = 0;
                    $saleInvoices = [];

                    if (is_array($invoiceNos)) {
                        foreach ($invoiceNos as $invoiceno) {
                            $sale = Sales::where('studentid', $student->id)->where('invoiceno', $invoiceno)
                                ->where('remaining_amount', '>', 0)
                                ->first();

                            if ($sale) {
                                $totalRemainingAmount += $sale->remaining_amount;
                                $saleInvoices[] = $sale->invoiceno;
                                $this->updateOtpForTheSale($sale, $otp);
                            }
                        }

                        if (empty($saleInvoices)) {
                            return response()->json([
                                'status' => false,
                                'statusCode' => 404,
                                'message' => 'Remaining sales not found',
                            ], 404);
                        }
                    }
                    break;
                case 'mainSale':
                    $mainSale = Sales::where('studentid', $student->id)
                        ->where('invoiceno', $request->invoiceNo)
                        ->first();
                    if ($mainSale) {
                        $this->updateOtpForTheSale($mainSale, $otp);
                    }
                    break;
                default:
                    return response()->json([
                        'status' => false,
                        'statusCode' => 400,
                        'message' => 'Invalid Sale Type',
                    ], 400);
            }

            // Send email
            Mail::send('emails.otp_template_for_sale', [
                'studentName' => $student->candidatename,
                'mobile' => $student->phone,
                'amount' => $request->amount,
                'invoiceNos' => $request->invoiceNo,
                'paymentType' => $request->type,
                'otp' => $otp,
            ], function ($message) use ($businessDetails) {
                $message->to($businessDetails->email);
                $message->subject('Sale OTP Verification');
            });

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'OTP shared successfully',
                'otp' => $otp
            ]);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => 'Student not Found',
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'Something went wrong',
                'error' => $e->getMessage(),
            ]);
        }
    }

    private function updateSale($sale, $paymentType, $amount = 0 , $remark)
    {
        if (empty($sale)) {
            throw new ModelNotFoundException('Sale verification failed');
        }

        $receiveAmount = $amount !== 0 ? $amount : $sale->remaining_amount;
        $remainingAmount = $sale->remaining_amount;
        $sale->receive_amount += $receiveAmount;

        if ($remainingAmount >= $receiveAmount) {
            $sale->remaining_amount = $remainingAmount - $receiveAmount;
            $sale->unused = 0;
        } else {
            $sale->remaining_amount = 0;
            $sale->unused = $receiveAmount - $remainingAmount;
        }

        $sale->payment_status = 1;
        $sale->tpdate = Carbon::now()->format('Y-m-d');
        $sale->save();

        // Now update Payment In sale
        $this->insertPaymentIn(Carbon::now()->format('Y-m-d'), $sale->studentid, $sale->invoiceno, $sale->remaining_amount, $receiveAmount, $paymentType, $remark ,false);
    }


    public function paidBillSbyTenant(Request $request)
    {
        try {
            DB::beginTransaction();

            // Validate the request data
            $validator = Validator::make($request->all(), [
                'studentId' => 'required|numeric',
                'saleType' => 'required|string|in:currentDue,totalDue,mainSale',
                'otp' => 'required|numeric',
                'paymentType' => 'required|in:cash,online',
                'amount' => 'required|numeric|min:1',
                'remark' => 'required|string'
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => "Validation error",
                    'error' => $validator->errors(),
                ], 422);
            }

            switch ($request->saleType) {
                case 'currentDue':
                    $sale = Sales::where('studentid', $request->studentId)
                        ->where('otp', $request->otp)
                        ->first();
                    $this->updateSale($sale, $request->paymentType, $request->amount,$request->remark);
                    break;
                case 'totalDue':
                    $sales = Sales::where('studentid', $request->studentId)->where('otp', $request->otp)->get();
                    if ($sales->isEmpty()) {
                        throw new ModelNotFoundException('Sales not found');
                    }
                    foreach ($sales as $sale) {
                        $this->updateSale($sale, $request->paymentType,$sale->remaining_amount,$request->remark);
                    }
                    break;
                case 'mainSale':
                    $sale = Sales::where('studentid', $request->studentId)->where('otp', $request->otp)->first();
                    $this->updateSale($sale, $request->paymentType, $request->amount,$request->remark);
                    break;
                default:
                    throw new ModelNotFoundException('Invalid Sale Type');
            }

            DB::commit();

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Payment processed successfully',
            ]);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json([
                'status' => false,
                'statusCode' => 400,
                'message' => "Otp Verification failed",
                'error' => $e->getMessage(),
            ], 400);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => "Something went wrong!",
                'error' => $e->getMessage(),
            ], 500);
        }
    }
}
