<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Carbon\Carbon;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use App\Models\empPaymentSetup;
use App\Models\EmpManagement;
use Exception;
use Illuminate\Support\Facades\DB;
use App\Models\EmpAttendance;
use App\Models\EmpSalary;
use Illuminate\Database\Eloquent\ModelNotFoundException;



class empPaymentManagementController
{

    private function changeSalaryInEmp($id, $salary)
    {
        try {
            // Find the employee by ID
            $employee = EmpManagement::findOrFail($id);

            // Update the current_salary field directly
            $employee->update([
                'current_salary' => $salary,
            ]);

            return true; // Indicate success
        } catch (\Exception $e) {
            // Log or handle the error as needed
            return false; // Indicate failure
        }
    }


    public function storePaymentSetup(Request $request, $empId)
    {
        $validator = Validator::make($request->all(), [
            'date'=> 'required|date',
            'monthly_payment' => 'required|numeric',
            'leave_allowed' => 'required|numeric',
        ]);

        if ($validator->fails()) {
            return back()->with('error', $validator->errors()->first());
        }

        try {
            // Check if the employee has an existing payment setup
            DB::beginTransaction();
            $employeePayment = new EmpPaymentSetup();
            $employeePayment->emp_id = $empId;
            $employeePayment->date = $request->input('date');
            $employeePayment->monthly_payment = $request->input('monthly_payment');
            $employeePayment->leave_allowed = $request->input('leave_allowed');
            $employeePayment->save();

            $result = $this->changeSalaryInEmp($empId, $request->input('monthly_payment'));

            if (!$result) {
                throw new \Exception('Error updating employee salary.');
            }
            // Change salary in employees table as well
            DB::commit();
            return back()->with(['success' => "Payment setup created successfully"]);
        } catch (\Exception $e) {
            DB::rollBack();
            return back()->with('error', 'Error saving payment setup: ' . $e->getMessage());
        }
    }

    public function getMonthlyPayment(Request $request, $empId)
    {
        try {
            // Get the selected date and extract month and year
            $selectedDate = $request->changeDate;
            $selectedMonth = Carbon::parse($selectedDate)->month;
            $selectedYear = Carbon::parse($selectedDate)->year;

            // Calculate the first and last day of the selected month
            $firstDayOfMonth = Carbon::parse($selectedDate)->startOfMonth();
            $lastDayOfMonth = Carbon::parse($selectedDate)->endOfMonth();

            // Get the total days in the selected month
            $totalDaysInMonth = $lastDayOfMonth->diffInDays($firstDayOfMonth) + 1;

            // Fetch the salary setup data for the employee
            $salarySetupData = EmpPaymentSetup::where('emp_id', $empId)
                ->orderByDesc('id')
                ->first();

            if (!$salarySetupData) {
                return response()->json([
                    'status' => false,
                    'message' => 'Payment setup not found.',
                ], 404);
            }

            // Extract necessary details from the salary setup
            $leaveAllowed = $salarySetupData->leave_allowed;
            $selectedMonthsSalary = $salarySetupData->monthly_payment;

            // Calculate attendance metrics for the current month
            $attendanceData = EmpAttendance::where('employee_id', $empId)
                ->whereBetween('date', [$firstDayOfMonth, $lastDayOfMonth])
                ->get();

            // Initialize counters
            $totalPresent = 0;
            $totalAbsent = 0;
            $totalHalfDay = 0;
            $totalLeave = 0;

            // Process each attendance record
            foreach ($attendanceData as $attendance) {
                switch ($attendance->status) {
                    case 'P':
                        $totalPresent++;
                        break;
                    case 'A':
                        $totalAbsent++;
                        break;
                    case 'H':
                        $totalHalfDay++;
                        break;
                    case 'L':
                        $totalLeave++;
                        break;
                    default:
                        // Handle unrecognized status if needed
                        break;
                }
            }
            // Adjust totalPresent based on half-days, absent days, and leave days
            $totalPresent += $totalHalfDay / 2;

            // Calculate the actual leave days based on the allowed limit
            if ($totalLeave > $leaveAllowed) {
                $actualLeave = $totalLeave - $leaveAllowed;
            } else {
                $actualLeave = 0;
            }

            // Deduct excess leave days beyond the allowed limit from totalPresent
            $totalPresent -= $totalAbsent + $actualLeave;

            // Ensure totalPresent is not negative
            $totalPresent = max($totalPresent, 0);

            // Calculate total salary based on present days
            $totalDaysInMonth = $lastDayOfMonth->diffInDays($firstDayOfMonth) + 1;
            $totalSalary = ($selectedMonthsSalary / $totalDaysInMonth) * $totalPresent;

            return response()->json([
                'totalPresent' => round(max($totalPresent, 0), 2),
                'totalSalary' =>  round(max($totalSalary, 0), 2),
                'paymentDetails' => $this->getPaymentDetails($empId, $selectedDate),
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Error processing payment details.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    private function  getPaymentDetails($empId, $slectedMonth)
    {
        $slectedMonth = Carbon::parse($slectedMonth)->format('Y-m-d');
        $checkData = EmpSalary::where('emp_id', $empId)->where('salary_month', $slectedMonth)->first();
        return  [
            'data' => $checkData,
            'paymentStatus' => empty($checkData) ? 'Unpaid' : 'Paid'
        ];
    }

    public function makeUserPaymentMonthly(Request $request, $empId)
    {
        $validator = Validator::make($request->all(), [
            'salary_month' => 'required|date',
            'pay_date' => 'required|date',
            'total_present' => 'required|numeric',
            'amount' => 'required|numeric',
            'bonus' => 'nullable|numeric',
            'penalty' => 'nullable|numeric',
            'total_amount' => 'required|numeric',
            'payment_type' => 'required|string',
        ]);

        if ($validator->fails()) {
            return back()->with(['error' => $validator->errors()->first()]);
        }
        // check if the salary all ready  paid for this month or not
        $salaryMonth = Carbon::parse($request->salary_month)->format('Y-m-d');
        $isPaid = EmpSalary::where('salary_month', $salaryMonth)->where('emp_id', $empId)->first();
        if (!empty($isPaid)) {
            return back()->with(['error' => 'Salary already created  for this Month']);
        }
        try {
            $data = $validator->validated();
            $data['emp_id'] = $empId;
            $data['salary_month'] = $salaryMonth;
            EmpSalary::create($data);
            return back()->with('success', 'User payment record created successfully.');
        } catch (Exception $e) {
            return back()->with(['error' => 'Somthing Went Wronge']);
        }
    }

    public function checkSalaryPaid(Request $request)
    {
        try {
            $empId = $request->userId;
            $slectedMonth = $request->selectedMonth;
            $data =  $this->getPaymentDetails($empId, $slectedMonth);
            return response()->json($data);
        } catch (Exception $e) {
            return response()->json([
                'message' => 'Somthing went wronge',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function updatesalary(Request $request, $empId)
    {
        $validator = Validator::make($request->all(), [
            'emp_id' => 'required',
            'salary_month' => 'required|date',
            'pay_date' => 'required|date',
            'total_present' => 'required|numeric',
            'amount' => 'required|numeric',
            'bonus' => 'nullable|numeric',
            'penalty' => 'nullable|numeric',
            'total_amount' => 'required|numeric',
            'payment_type' => 'required|string',
        ]);

        if ($validator->fails()) {
            return back()->with(['error' => $validator->errors()->first()]);
        }
        // check if the salary all ready  paid for this month or not
        $salaryMonth = Carbon::parse($request->salary_month)->format('Y-m-d');

        try {
            $isPaid = EmpSalary::findOrFail($empId);
            $data = $validator->validated();
            $data['salary_month'] = $salaryMonth;
            $isPaid->update($data);
            return back()->with('success', 'User payment record updated successfully.');
        } catch (ModelNotFoundException $e) {
            return back()->with()(['error' => 'salary Records not Found']);
        } catch (Exception $e) {
            return back()->with(['error' => 'Somthing Went Wronge']);
        }
    }

    public function deleteSalary($id)
    {
        try {
            $delete = EmpSalary::findOrFail($id);
            $delete->delete();
            return response()->json([
                "status" => true,
                "message" => "Successfully Deleted!"
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'message' => 'Record Not Found',
                'error' => $e->getMessage()
            ], 500);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Somthing Went Wronge',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function getPaymentSetupData(Request $request)
    {
        try {
            $selectedData = $request->selectedData;
            $data = empPaymentSetup::where('date', $selectedData)->first();

            if (empty($data)) {
                return response()->json([
                    'status' => false,
                    'message' => 'No Data Available!'
                ], 404);
            }
            return response()->json([
                'status' => true,
                'data' => [
                    'monthly_payment' => $data->monthly_payment,
                    'leave_allowed' => $data->leave_allowed,
                ],
                'message' => 'salaray setup data'
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'somthing went wronge',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
}
